@@ -29,8 +29,8 @@ Box_DB::Box_DB()
 : m_Box_Buttons(false, 6),
   m_Button_Cancel(Gtk::Stock::CANCEL)
 {
-  m_pDocument = 0;
-  
+  //m_pDocument = 0;
+
   set_border_width(6);
   set_spacing(6);
 
@@ -43,7 +43,7 @@ Box_DB::Box_DB(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& /
   m_Box_Buttons(false, 6),
   m_Button_Cancel(Gtk::Stock::CANCEL)
 {
-  m_pDocument = 0;
+  //m_pDocument = 0;
 
   set_border_width(6);
   set_spacing(6);
@@ -63,29 +63,12 @@ Box_DB::~Box_DB()
 {
 }
 
-
-
 void Box_DB::on_Button_Cancel()
 {
   //Tell the parent dialog that the user has clicked [Cancel]:
   signal_cancelled.emit();
 }
 
-
-
-void Box_DB::hint_set(const Glib::ustring& /* strText */)
-{
-  //This method will only succeed *after* the widget has been added to its parent widget.
-
-  Gtk::Window* pApp = get_app_window();
-
-  App_Glom* pAppGlom = dynamic_cast<App_Glom*>(pApp);
-  if(pAppGlom)
-  {
-    //Disable this for now. It looks annoying. //pAppGlom->statusbar_set_text(strText);
-  }
-}
-
 const Gtk::Window* Box_DB::get_app_window() const
 {
   Box_DB* nonconst = const_cast<Box_DB*>(this);
@@ -94,6 +77,9 @@ const Gtk::Window* Box_DB::get_app_window() const
   
 Gtk::Window* Box_DB::get_app_window()
 {
+  return dynamic_cast<Gtk::Window*>(get_toplevel());
+/*
+
   Gtk::Widget* pWidget = get_parent();
   while(pWidget)
   {
@@ -112,12 +98,15 @@ Gtk::Window* Box_DB::get_app_window()
   }
 
   return 0; //not found.
+*/
 }
 
+/*
 void Box_DB::show_hint()
 {
   hint_set(m_strHint);
 }
+*/
 
 void Box_DB::set_button_cancel(Gtk::Button& button)
 {
