@@ -63,6 +63,7 @@
 #define GLOM_ATTRIBUTE_OTHER_TABLE "other_table"
 #define GLOM_ATTRIBUTE_OTHER_KEY "other_key"
 #define GLOM_ATTRIBUTE_AUTO_CREATE "auto_create"
+#define GLOM_ATTRIBUTE_ALLOW_EDIT "allow_edit"
 
 #define GLOM_NODE_GROUPS "groups"
 #define GLOM_NODE_GROUP "group"
@@ -293,28 +294,32 @@ void Document_Glom::remove_table(const Glib::ustring& table_name)
   for(type_tables::iterator iter = m_tables.begin(); iter != m_tables.end(); ++iter)
   {
     DocumentTableInfo& info = iter->second;
-
-    bool something_changed = true;
-    type_vecRelationships::iterator iterRel = info.m_relationships.begin();
-    while(something_changed)
+    
+    if(!(info.m_relationships.empty()))
     {
-      if(iterRel->get_to_table() == table_name)
+      type_vecRelationships::iterator iterRel = info.m_relationships.begin();
+      bool something_changed = true;
+      while(something_changed && !info.m_relationships.empty())
       {
-        //Loop again, because we have changed the structure:
-        remove_relationship(*iterRel); //Also removes anything that uses the relationship.
-
-        something_changed = true;
-        iterRel = info.m_relationships.begin();
-      }
-      else
-      {
-        if(iterRel == info.m_relationships.end())
-          something_changed = false; //We've looked at them all, without changing things.
+        if(iterRel->get_to_table() == table_name)
+        {
+          //Loop again, because we have changed the structure:
+          remove_relationship(*iterRel); //Also removes anything that uses the relationship.
+  
+          something_changed = true;
+          iterRel = info.m_relationships.begin();
+        }
+        else
+        {
+          ++iterRel;
+          
+          if(iterRel == info.m_relationships.end())
+            something_changed = false; //We've looked at them all, without changing things.
+        }
       }
     }
 
   }
-
 }
 
 
@@ -392,8 +397,9 @@ void Document_Glom::change_field_name(const Glib::ustring& table_name, const Gli
       //Change it:
       iterFind->set_name(strFieldNameNew);
     }
+    
 
-    //Find any relationships or layouts that use this field
+    //Find any relationships, layouts, or formatting that use this field
     //Look at each table:
     for(type_tables::iterator iter = m_tables.begin(); iter != m_tables.end(); ++iter)
     {
@@ -418,18 +424,42 @@ void Document_Glom::change_field_name(const Glib::ustring& table_name, const Gli
           }
         }
       }
-    }
 
-    //Look at each layout:
-    //TODO: Remember to change it in other layouts when we add the ability to show fields from other tables.
-    for(DocumentTableInfo::type_layouts::iterator iterLayouts = iterFindTable->second.m_layouts.begin(); iterLayouts != iterFindTable->second.m_layouts.end(); ++iterLayouts)
-    {
-      //Look at each group:
-      for(type_mapLayoutGroupSequence::iterator iterGroup = iterLayouts->m_layout_groups.begin(); iterGroup != iterLayouts->m_layout_groups.end(); ++iterGroup)
+      //Look at all field formatting:
+      for(type_vecFields::iterator iterFields = iter->second.m_fields.begin(); iterFields != iter->second.m_fields.end(); ++iterFields)
+      {
+        iterFields->m_default_formatting.change_field_name(table_name, strFieldNameOld, strFieldNameNew);
+      }
+           
+      
+      const bool is_parent_table = (iter->second.m_info.get_name() == table_name);
+        
+      //Look at each layout:
+      for(DocumentTableInfo::type_layouts::iterator iterLayouts = iter->second.m_layouts.begin(); iterLayouts != iter->second.m_layouts.end(); ++iterLayouts)
+      {
+        
+        //Look at each group:
+        for(type_mapLayoutGroupSequence::iterator iterGroup = iterLayouts->m_layout_groups.begin(); iterGroup != iterLayouts->m_layout_groups.end(); ++iterGroup)
+        {
+          //Change the field if it is in this group:
+          if(is_parent_table)
+            iterGroup->second.change_field_item_name(table_name, strFieldNameOld, strFieldNameNew);
+          else
+            iterGroup->second.change_related_field_item_name(table_name, strFieldNameOld, strFieldNameNew);
+        }
+      }
+      
+      
+      //Look at each report:
+      for(DocumentTableInfo::type_reports::iterator iterReports = iter->second.m_reports.begin(); iterReports != iter->second.m_reports.end(); ++iterReports)
       {
         //Change the field if it is in this group:
-        iterGroup->second.change_field_item_name(strFieldNameOld, strFieldNameNew);
+        if(is_parent_table)
+          iterReports->second.m_layout_group.change_field_item_name(table_name, strFieldNameOld, strFieldNameNew);
+        else
+          iterReports->second.m_layout_group.change_related_field_item_name(table_name, strFieldNameOld, strFieldNameNew);
       }
+      
     }
    
     set_modified();
@@ -479,6 +509,60 @@ void Document_Glom::change_table_name(const Glib::ustring& strTableNameOld, cons
   }
 }
 
+void Document_Glom::change_relationship_name(const Glib::ustring& table_name, const Glib::ustring& name, const Glib::ustring& name_new)
+{
+  type_tables::iterator iterFindTable = m_tables.find(table_name);
+  if(iterFindTable != m_tables.end())
+  {
+    //Change the relationship name:
+    type_vecRelationships::iterator iterRelFind = std::find_if( iterFindTable->second.m_relationships.begin(), iterFindTable->second.m_relationships.end(), predicate_FieldHasName<Relationship>(name) );
+    if(iterRelFind != iterFindTable->second.m_relationships.end())
+      iterRelFind->set_name(name_new);
+      
+      
+    //Find any layouts or reports that use this field
+    //Look at each table:
+    for(type_tables::iterator iter = m_tables.begin(); iter != m_tables.end(); ++iter)
+    {
+       //Look at all field formatting:
+      for(type_vecFields::iterator iterFields = iter->second.m_fields.begin(); iterFields != iter->second.m_fields.end(); ++iterFields)
+      {
+        iterFields->m_default_formatting.change_relationship_name(table_name, name, name_new);
+      }
+      
+      const bool is_parent_table = (iter->second.m_info.get_name() == table_name);
+        
+      //Look at each layout:
+      for(DocumentTableInfo::type_layouts::iterator iterLayouts = iter->second.m_layouts.begin(); iterLayouts != iter->second.m_layouts.end(); ++iterLayouts)
+      {
+        //Look at each group:
+        for(type_mapLayoutGroupSequence::iterator iterGroup = iterLayouts->m_layout_groups.begin(); iterGroup != iterLayouts->m_layout_groups.end(); ++iterGroup)
+        {
+          //Change the field if it is in this group:
+          if(is_parent_table)
+            iterGroup->second.change_relationship_name(table_name, name, name_new);
+          else
+            iterGroup->second.change_related_relationship_name(table_name, name, name_new);
+        }
+      }
+      
+      
+      //Look at each report:
+      for(DocumentTableInfo::type_reports::iterator iterReports = iter->second.m_reports.begin(); iterReports != iter->second.m_reports.end(); ++iterReports)
+      {
+        //Change the field if it is in this group:
+        if(is_parent_table)
+          iterReports->second.m_layout_group.change_relationship_name(table_name, name, name_new);
+        else
+          iterReports->second.m_layout_group.change_related_relationship_name(table_name, name, name_new);
+      }
+      
+    }
+   
+    set_modified();
+  }
+ }
+  
 
 
 bool Document_Glom::get_node_attribute_value_as_bool(const xmlpp::Element* node, const Glib::ustring& strAttributeName)
@@ -591,12 +675,6 @@ void Document_Glom::set_tables(const type_listTableInfo& tables)
 
 }
 
-Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_relationship_data_layout_groups_plus_new_fields(const Glib::ustring& layout_name, const Relationship& relationship) const
-{
-  //TODO: Use an actual relationship_name instead of concatenating:
-  return get_data_layout_groups_plus_new_fields(layout_name + "_related_" + relationship.get_name(), relationship.get_from_table(), relationship.get_to_table());
-}
-
 void Document_Glom::fill_layout_field_details(const Glib::ustring& parent_table_name, LayoutGroup& layout_group) const
 {
   //Get the full field information for the LayoutItem_Fields in this group:
@@ -608,13 +686,22 @@ void Document_Glom::fill_layout_field_details(const Glib::ustring& parent_table_
     LayoutItem_Field* layout_field = dynamic_cast<LayoutItem_Field*>(layout_item);
     if(layout_field)
     {
-      get_field(parent_table_name, layout_field->get_name(), layout_field->m_field);
+      if(layout_field->get_has_relationship_name()) //If it is a related field, instead of a field in parent_table_name
+        get_field(layout_field->m_relationship.get_to_table(), layout_field->get_name(), layout_field->m_field);
+      else  
+        get_field(parent_table_name, layout_field->get_name(), layout_field->m_field);
     }
     else
     {
-      LayoutGroup* layout_group_child = dynamic_cast<LayoutGroup*>(layout_item);
-      if(layout_group_child)
-        fill_layout_field_details(parent_table_name, *layout_group_child); //recurse
+      LayoutItem_Portal* layout_portal_child = dynamic_cast<LayoutItem_Portal*>(layout_item);
+      if(layout_portal_child)
+        fill_layout_field_details(layout_portal_child->m_relationship.get_to_table(), *layout_portal_child); //recurse
+      else
+      {  
+        LayoutGroup* layout_group_child = dynamic_cast<LayoutGroup*>(layout_item);
+        if(layout_group_child)
+          fill_layout_field_details(parent_table_name, *layout_group_child); //recurse
+      }
     }
   }
 }
@@ -627,13 +714,9 @@ void Document_Glom::fill_layout_field_details(const Glib::ustring& parent_table_
   }
 }
 
-Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups_plus_new_fields(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name, const Glib::ustring& table_name) const
+Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups_plus_new_fields(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name) const
 {
-  Glib::ustring child_table_name = table_name;
-  if(child_table_name.empty())
-    child_table_name = parent_table_name;
-
-  type_mapLayoutGroupSequence result = get_data_layout_groups(layout_name, parent_table_name, child_table_name);
+  type_mapLayoutGroupSequence result = get_data_layout_groups(layout_name, parent_table_name);
 
 
   //If there are no fields in the layout, then add a default:
@@ -644,11 +727,13 @@ Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups
 
   if(create_default)
   {
-    g_warning("Document_Glom::get_data_layout_groups_plus_new_fields(): Creating default layout for table %s (child_table=%s), for layout %s", parent_table_name.c_str(), child_table_name.c_str(), layout_name.c_str());
+    //g_warning("Document_Glom::get_data_layout_groups_plus_new_fields(): Creating default layout for table %s (child_table=%s), for layout %s", parent_table_name.c_str(), child_table_name.c_str(), layout_name.c_str());
 
     //Get the last top-level group. We will add new fields into this one:
     //TODO_Performance: There must be a better way to do this:
     LayoutGroup* pTopLevel = 0;
+    LayoutGroup* pOverview = 0; //The default layout has a main group with an overview and details group inside.
+    LayoutGroup* pDetails = 0;
     for(type_mapLayoutGroupSequence::iterator iterGroups = result.begin(); iterGroups != result.end(); ++iterGroups)
     {
       pTopLevel = &(iterGroups->second);
@@ -663,10 +748,33 @@ Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups
       group.m_columns_count = 1;
       result[1] = group;
       pTopLevel = &(result[1]);
+      
+      if(layout_name == "details") //The Details default layut is a bit more complicated.
+      {
+        LayoutGroup overview;
+        overview.set_name("overview");
+        overview.m_title = "Overview"; //Don't translate this, but TODO: add standard translations.
+        overview.m_columns_count = 2;
+        pOverview = dynamic_cast<LayoutGroup*>(pTopLevel->add_item(overview));
+        
+        LayoutGroup details;
+        details.set_name("details");
+        details.m_title = "Details"; //Don't translate this, but TODO: add standard translations.
+        details.m_columns_count = 2;
+        pDetails = dynamic_cast<LayoutGroup*>(pTopLevel->add_item(details));
+      }
     }
 
+    //If, for some reason, we didn't create the-subgroups, add everything to the top level group:
+    if(!pOverview)
+      pOverview = pTopLevel;
+      
+    if(!pDetails)
+      pDetails = pTopLevel;
+      
+    
     //Discover new fields, and add them:
-    type_vecFields all_fields = get_table_fields(child_table_name);
+    type_vecFields all_fields = get_table_fields(parent_table_name);
     for(type_vecFields::const_iterator iter = all_fields.begin(); iter != all_fields.end(); ++iter)
     {
       const Glib::ustring field_name = iter->get_name();
@@ -691,8 +799,10 @@ Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups
           //layout_item.set_table_name(child_table_name); //TODO: Allow viewing of fields through relationships.
           //layout_item.m_sequence = sequence;  add_item() will fill this.
 
-
-          pTopLevel->add_item(layout_item);
+          if(layout_item.m_field.get_primary_key())
+            pOverview->add_item(layout_item);
+          else
+            pDetails->add_item(layout_item);
         }
       }
     }
@@ -701,19 +811,15 @@ Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups
   return result;  
 }
   
-Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name, const Glib::ustring& table_name) const
+Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name) const
 {
-  Glib::ustring child_table = table_name;
-  if(child_table.empty())
-    child_table = parent_table_name;
-
   type_tables::const_iterator iterFind = m_tables.find(parent_table_name);
   if(iterFind != m_tables.end())
   {
     const DocumentTableInfo& info = iterFind->second;
 
     //Look for the layout with this name:
-    DocumentTableInfo::type_layouts::const_iterator iter = std::find_if(info.m_layouts.begin(), info.m_layouts.end(), predicate_Layout<LayoutInfo>(child_table, layout_name));
+    DocumentTableInfo::type_layouts::const_iterator iter = std::find_if(info.m_layouts.begin(), info.m_layouts.end(), predicate_Layout<LayoutInfo>(parent_table_name, layout_name));
     if(iter != info.m_layouts.end())
     {
       return iter->m_layout_groups; //found
@@ -723,17 +829,9 @@ Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups
   return type_mapLayoutGroupSequence(); //not found
 }
 
-void Document_Glom::set_relationship_data_layout_groups(const Glib::ustring& layout_name, const Relationship& relationship, const type_mapLayoutGroupSequence& groups)
+void Document_Glom::set_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name, const type_mapLayoutGroupSequence& groups)
 {
-  //TODO: Use an actual relationship_name instead of concatenating:
-  set_data_layout_groups(layout_name + "_related_" + relationship.get_name(), relationship.get_from_table(), groups, relationship.get_to_table());
-}
-
-void Document_Glom::set_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name, const type_mapLayoutGroupSequence& groups, const Glib::ustring& table_name)
-{
-  Glib::ustring child_table_name = table_name;
-  if(child_table_name.empty())
-    child_table_name = parent_table_name;
+  Glib::ustring child_table_name = parent_table_name; //TODO: Remove this cruft.
 
   //g_warning("Document_Glom::set_data_layout_groups(): ADDING layout for table %s (child_table=%s), for layout %s", parent_table_name.c_str(), child_table_name.c_str(), layout_name.c_str());
 
@@ -800,6 +898,15 @@ bool Document_Glom::get_table_is_known(const Glib::ustring& table_name) const
   return (iterFind != m_tables.end());
 }
 
+bool Document_Glom::get_table_is_hidden(const Glib::ustring& table_name) const
+{
+  type_tables::const_iterator iterFind = m_tables.find(table_name);
+  if(iterFind != m_tables.end())
+    return iterFind->second.m_info.m_hidden;
+  else
+    return false; //It's not even known.
+}
+
 AppState::userlevels Document_Glom::get_userlevel() const
 {
   userLevelReason reason;
@@ -874,6 +981,15 @@ Glib::ustring Document_Glom::get_default_table() const
   return Glib::ustring();
 }
 
+Glib::ustring Document_Glom::get_first_table() const
+{
+  if(m_tables.empty())
+    return Glib::ustring();
+    
+  type_tables::const_iterator iter = m_tables.begin();
+  return iter->second.m_info.m_name;
+}
+
 void Document_Glom::set_modified(bool value)
 {
   if(value && m_block_modified_set)
@@ -1017,11 +1133,12 @@ void Document_Glom::load_after_layout_group(const xmlpp::Element* node, const Gl
       }
       else if(element->get_name() == GLOM_NODE_DATA_LAYOUT_PORTAL)
       {
-        LayoutItem_Portal item;
-        item.set_relationship( get_node_attribute_value(element, GLOM_ATTRIBUTE_RELATIONSHIP_NAME) );
+        LayoutItem_Portal portal;
+        portal.set_relationship( get_node_attribute_value(element, GLOM_ATTRIBUTE_RELATIONSHIP_NAME) );
 
-        item.m_sequence = sequence;
-        group.add_item(item, sequence);
+        //Recurse:
+        load_after_layout_group(element, table_name, portal);
+        group.add_item(portal);
       }
       else if(element->get_name() == GLOM_NODE_DATA_LAYOUT_ITEM_GROUPBY)
       {
@@ -1195,6 +1312,7 @@ bool Document_Glom::load_after()
                 relationship.set_to_table( get_node_attribute_value(nodeChild, GLOM_ATTRIBUTE_OTHER_TABLE) );
                 relationship.set_to_field( get_node_attribute_value(nodeChild, GLOM_ATTRIBUTE_OTHER_KEY) );
                 relationship.set_auto_create( get_node_attribute_value_as_bool(nodeChild, GLOM_ATTRIBUTE_AUTO_CREATE) );
+                 relationship.set_allow_edit( get_node_attribute_value_as_bool(nodeChild, GLOM_ATTRIBUTE_ALLOW_EDIT) );
 
                 doctableinfo.m_relationships.push_back(relationship);
               }
@@ -1425,7 +1543,16 @@ void Document_Glom::save_before_layout_group(xmlpp::Element* node, const LayoutG
     }
     else
     {
-      child = node->add_child(GLOM_NODE_DATA_LAYOUT_GROUP);
+      const LayoutItem_Portal* portal = dynamic_cast<const LayoutItem_Portal*>(&group);
+      if(portal) //If it is a related records portal
+      {
+        child = node->add_child(GLOM_NODE_DATA_LAYOUT_PORTAL);
+        child->set_attribute(GLOM_ATTRIBUTE_RELATIONSHIP_NAME, portal->get_relationship());
+      }
+      else
+      {
+        child = node->add_child(GLOM_NODE_DATA_LAYOUT_GROUP);
+      }
     }
   }
 
@@ -1443,8 +1570,8 @@ void Document_Glom::save_before_layout_group(xmlpp::Element* node, const LayoutG
     //g_warning("save_before_layout_group: child part type=%s", item->get_part_type_name().c_str());
 
     const LayoutGroup* child_group = dynamic_cast<const LayoutGroup*>(item);
-    if(child_group) //If it is a group
-    {
+    if(child_group) //If it is a group, portal, summary, or groupby.
+    {   
       //recurse:
       save_before_layout_group(child, *child_group);
     }
@@ -1465,19 +1592,7 @@ void Document_Glom::save_before_layout_group(xmlpp::Element* node, const LayoutG
           xmlpp::Element* nodeItem = child->add_child(GLOM_NODE_DATA_LAYOUT_ITEM);
           save_before_layout_item_field(nodeItem, *field);
         }
-        else
-        {
-          const LayoutItem_Portal* portal = dynamic_cast<const LayoutItem_Portal*>(item);
-          if(portal) //If it is a portal
-          {
-            xmlpp::Element* nodeItem = child->add_child(GLOM_NODE_DATA_LAYOUT_PORTAL);
-            nodeItem->set_attribute(GLOM_ATTRIBUTE_RELATIONSHIP_NAME, portal->get_relationship());
-
-            set_node_attribute_value_as_decimal(nodeItem, GLOM_ATTRIBUTE_SEQUENCE, item->m_sequence);
-          }
-        }
       }
-
     }
 
     //g_warning("save_before_layout_group: after child part type=%s", item->get_part_type_name().c_str());
@@ -1575,6 +1690,7 @@ bool Document_Glom::save_before()
           set_node_attribute_value(elemRelationship, GLOM_ATTRIBUTE_OTHER_TABLE, relationship.get_to_table());
           set_node_attribute_value(elemRelationship, GLOM_ATTRIBUTE_OTHER_KEY, relationship.get_to_field());
           set_node_attribute_value_as_bool(elemRelationship, GLOM_ATTRIBUTE_AUTO_CREATE, relationship.get_auto_create());
+          set_node_attribute_value_as_bool(elemRelationship, GLOM_ATTRIBUTE_ALLOW_EDIT, relationship.get_allow_edit());
         }
 
 
@@ -1611,7 +1727,9 @@ bool Document_Glom::save_before()
           xmlpp::Element* nodeGroups = nodeReport->add_child(GLOM_NODE_DATA_LAYOUT_GROUPS);
           save_before_layout_group(nodeGroups, iter->second.m_layout_group);
         }
-
+        
+        
+        nodeTable->add_child_text("\n\n"); //Make it a bit easier to read, 
       }
 
     } //for m_tables
@@ -1635,7 +1753,7 @@ bool Document_Glom::save_before()
       nodeGroup->set_attribute(GLOM_ATTRIBUTE_NAME, group_info.m_name);
       set_node_attribute_value_as_bool(nodeGroup, GLOM_ATTRIBUTE_DEVELOPER, group_info.m_developer);
 
-      //The privilieges for each table, for this group:
+      //The privileges for each table, for this group:
       for(GroupInfo::type_map_table_privileges::const_iterator iter = group_info.m_map_privileges.begin(); iter != group_info.m_map_privileges.end(); ++iter)
       {
         xmlpp::Element* nodeTablePrivs = nodeGroups->add_child(GLOM_NODE_TABLE_PRIVS);
@@ -1756,19 +1874,10 @@ void Document_Glom::update_cached_relationships(LayoutGroup& group, const Glib::
       LayoutItem_Portal* pPortal = dynamic_cast<LayoutItem_Portal*>(pItem);
       if(pPortal)
       {
-
         get_relationship(table_name, pPortal->get_relationship(), pPortal->m_relationship);
 
-        //Update the Portal's group: TODO: Do this in place, instead of using set/get.
-        const Glib::ustring layout_name = "list_related"; //TODO: This is silly.
-        type_mapLayoutGroupSequence portalGroups = get_relationship_data_layout_groups_plus_new_fields(layout_name, pPortal->m_relationship);
-
-        for(type_mapLayoutGroupSequence::iterator iterGroup = portalGroups.begin(); iterGroup != portalGroups.end(); ++iterGroup)
-        {
-          update_cached_relationships(iterGroup->second, pPortal->m_relationship.get_to_table());
-        }
-
-        set_relationship_data_layout_groups(layout_name, pPortal->m_relationship, portalGroups);
+        //Update the Portal's group: TODO:
+        update_cached_relationships(*pPortal, pPortal->m_relationship.get_to_table());
       }
       else
       {
@@ -1898,4 +2007,97 @@ void Document_Glom::remove_report(const Glib::ustring& table_name, const Glib::u
   }
 }
 
+bool Document_Glom::get_relationship_is_to_one(const Glib::ustring& table_name, const Glib::ustring& relationship_name) const
+{
+  Relationship relationship;
+  bool found = get_relationship(table_name, relationship_name, relationship);
+  if(found)
+  {
+    Field field_to;
+    bool found = get_field(relationship.get_to_table(), relationship.get_to_field(), field_to);
+    if(found)
+      return (field_to.get_primary_key() || field_to.get_unique_key());
+  }
+  
+  return false;
+}
+  
+bool Document_Glom::get_field_used_in_relationship_to_one(const Glib::ustring& table_name, const Glib::ustring& field_name, Relationship& relationship) const
+{
+  //Initialize input parameter:
+  relationship = Relationship();
+  
+  type_tables::const_iterator iterFind = m_tables.find(table_name);
+  if(iterFind != m_tables.end())
+  {
+    //Look at each relationship:
+    for(type_vecRelationships::const_iterator iterRel = iterFind->second.m_relationships.begin(); iterRel != iterFind->second.m_relationships.end(); ++iterRel)
+    {
+      //If the relationship uses the field
+      if(iterRel->get_from_field() == field_name)
+      {
+        //if the to_table is not hidden:
+        if(!get_table_is_hidden(iterRel->get_to_table()))
+        {
+          //TODO_Performance: The use of this convenience method means we get the full relationship information again:
+          if(get_relationship_is_to_one(table_name, iterRel->get_name()))
+          {
+            relationship = *iterRel;
+            return true;
+          }
+        }
+      }
+    }
+  }
+  return false;
+}
+
+bool Document_Glom::get_field_used_in_relationship_to_one(const Glib::ustring& table_name, const Glib::ustring& field_name) const
+{
+  Relationship relationship; //ignored.
+  return get_field_used_in_relationship_to_one(table_name, field_name, relationship);
+}
+
+void Document_Glom::set_layout_record_viewed(const Glib::ustring& table_name, const Glib::ustring& layout_name, const Gnome::Gda::Value& primary_key_value)
+{
+  type_tables::iterator iterFind = m_tables.find(table_name);
+  if(iterFind != m_tables.end())
+  {
+    iterFind->second.m_map_current_record[layout_name] = primary_key_value;
+  }
+}
+
+Gnome::Gda::Value Document_Glom::get_layout_record_viewed(const Glib::ustring& table_name, const Glib::ustring& layout_name) const
+{
+  type_tables::const_iterator iterFind = m_tables.find(table_name);
+  if(iterFind != m_tables.end())
+  {
+    const DocumentTableInfo& info = iterFind->second;
+    DocumentTableInfo::type_map_layout_primarykeys::const_iterator iterLayoutKeys = info.m_map_current_record.find(layout_name);
+    if(iterLayoutKeys != info.m_map_current_record.end())
+      return iterLayoutKeys->second;
+  }
+  
+  return Gnome::Gda::Value(); //not found.
+}
+
+void Document_Glom::set_layout_current(const Glib::ustring& table_name, const Glib::ustring& layout_name)
+{
+  type_tables::iterator iterFind = m_tables.find(table_name);
+  if(iterFind != m_tables.end())
+  {
+    iterFind->second.m_layout_current = layout_name;
+  }
+}
+
+Glib::ustring Document_Glom::get_layout_current(const Glib::ustring& table_name) const
+{
+  type_tables::const_iterator iterFind = m_tables.find(table_name);
+  if(iterFind != m_tables.end())
+  {
+    return iterFind->second.m_layout_current;
+  }
+  
+  return Glib::ustring(); //not found.
+}
 
