@@ -76,13 +76,26 @@ public:
 
   typedef std::vector<Relationship> type_vecRelationships;
   virtual type_vecRelationships get_relationships(const Glib::ustring& table_name) const;
-  virtual void set_relationships(const Glib::ustring& table_name, const type_vecRelationships& vecRelationships);
+  void set_relationships(const Glib::ustring& table_name, const type_vecRelationships& vecRelationships);
 
-  virtual bool get_relationship(const Glib::ustring& table_name, const Glib::ustring& relationship_name, Relationship& relationship) const;
-  virtual void set_relationship(const Glib::ustring& table_name, const Relationship& relationship);
+  bool get_relationship(const Glib::ustring& table_name, const Glib::ustring& relationship_name, Relationship& relationship) const;
+  void set_relationship(const Glib::ustring& table_name, const Relationship& relationship);
 
-  virtual void remove_relationship(const Relationship& relationship);
+  void remove_relationship(const Relationship& relationship);
+  
+  /** Returns whether the relationship's to-field is a primary key  or unique field, meaning
+   * that there can be only one related record for each value of the from-field.
+   */
+  bool get_relationship_is_to_one(const Glib::ustring& table_name, const Glib::ustring& relationship_name) const;
+  
+  
+  /** Returns whether the field is the from-field in a to-one relationship.
+   * @see get_relationship_is_to_one(). Ignores hidden tables.
+   */
+  bool get_field_used_in_relationship_to_one(const Glib::ustring& table_name, const Glib::ustring& field_name) const;
+  bool get_field_used_in_relationship_to_one(const Glib::ustring& table_name, const Glib::ustring& field_name, Relationship& relationship) const;
 
+    
   typedef std::vector<Field> type_vecFields;
   virtual type_vecFields get_table_fields(const Glib::ustring& table_name) const;
   virtual void set_table_fields(const Glib::ustring& table_name, const type_vecFields& vecFields);
@@ -93,9 +106,7 @@ public:
   typedef std::map<guint, LayoutGroup> type_mapLayoutGroupSequence;
   virtual type_mapLayoutGroupSequence get_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name) const;
 
-  virtual void set_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name, const type_mapLayoutGroupSequence& groups, const Glib::ustring& table_name = Glib::ustring());
-
-  virtual void set_relationship_data_layout_groups(const Glib::ustring& layout_name, const Relationship& relationship, const type_mapLayoutGroupSequence& groups);
+  virtual void set_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name, const type_mapLayoutGroupSequence& groups);
 
   /**
    * @para The layout_name, such as "details", "list".
@@ -127,6 +138,7 @@ public:
   virtual void remove_table(const Glib::ustring& table_name);
 
   virtual bool get_table_is_known(const Glib::ustring& table_name) const;
+  virtual bool get_table_is_hidden(const Glib::ustring& table_name) const;
 
   virtual Glib::ustring get_table_title(const Glib::ustring& table_name) const;
   virtual void set_table_title(const Glib::ustring& table_name, const Glib::ustring& value);
@@ -156,6 +168,12 @@ public:
   void set_report(const Glib::ustring& table_name, const Report& report);
   bool get_report(const Glib::ustring& table_name, const Glib::ustring& report_name, Report& report) const;
   void remove_report(const Glib::ustring& table_name, const Glib::ustring& report_name);
+  
+  void set_layout_record_viewed(const Glib::ustring& table_name, const Glib::ustring& layout_name, const Gnome::Gda::Value& primary_key_value);
+  Gnome::Gda::Value get_layout_record_viewed(const Glib::ustring& table_name, const Glib::ustring& layout_name) const;
+  
+  void set_layout_current(const Glib::ustring& table_name, const Glib::ustring& layout_name);
+  Glib::ustring get_layout_current(const Glib::ustring& table_name) const;
 
   enum userLevelReason
   {
@@ -241,6 +259,12 @@ protected:
 
     typedef std::map<Glib::ustring, Report> type_reports; //map of report names to reports
     type_reports m_reports;
+    
+    //Per-session, not saved in document:
+    typedef std::map<Glib::ustring, Gnome::Gda::Value> type_map_layout_primarykeys;
+    type_map_layout_primarykeys m_map_current_record; //The record last viewed in each layout.
+    
+    Glib::ustring m_layout_current;
   };
 
   DocumentTableInfo& get_table_info_with_add(const Glib::ustring& table_name);
