@@ -20,11 +20,11 @@
 
 #include "box_data.h"
 #include "../data_structure/glomconversions.h"
+#include "../utils.h"
 #include "../data_structure/layout/layoutitem_field.h"
 #include "../python_embed/glom_python.h"
 #include <bakery/App/App_Gtk.h> //For util_bold_message().
 #include <algorithm> //For std::find()
-#include <libxslt/transform.h>
 #include "config.h"
 #include <glibmm/i18n.h>
 
@@ -33,45 +33,51 @@ Box_Data::Box_Data()
   m_pDialogLayout(0)
 {
   m_bUnstoredData = false;
-   
 
-    
   //Connect signals:
   m_Button_Find.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data::on_Button_Find));
 }
 
 Box_Data::~Box_Data()
 {
+  if(m_pDialogLayout)
+  {
+    remove_view(m_pDialogLayout);
+    delete m_pDialogLayout;
+  }
 }
 
-void Box_Data::init_db_details(const Glib::ustring& strTableName, const Glib::ustring& strWhereClause)
+bool Box_Data::init_db_details(const Glib::ustring& strTableName, const Glib::ustring& strWhereClause)
+{
+  m_strTableName = strTableName;
+  m_strWhereClause = strWhereClause;
+
+  create_layout(); //So that fill_from_database() can succeed.
+
+  return Box_DB_Table::init_db_details(strTableName); //Calls fill_from_database().
+}
+
+bool Box_Data::refresh_data_from_database(const Glib::ustring& strWhereClause)
 {
   m_strWhereClause = strWhereClause;
 
-  Box_DB_Table::init_db_details(strTableName);
+  return Box_DB_Table::refresh_data_from_database(); //Calls fill_from_database().
 }
 
-void Box_Data::refresh_db_details(const Glib::ustring& strWhereClause)
-{
-  m_strWhereClause = strWhereClause;
-
-  Box_DB_Table::refresh_db_details();
-}
-
-Glib::ustring Box_Data::get_WhereClause() const
+Glib::ustring Box_Data::get_find_where_clause() const
 {
   Glib::ustring strClause;
 
   //Look at each field entry and build e.g. 'Name = "Bob"'
-  for(type_vecLayoutFields::const_iterator iter = m_Fields.begin(); iter != m_Fields.end(); ++iter)
+  for(type_vecLayoutFields::const_iterator iter = m_FieldsShown.begin(); iter != m_FieldsShown.end(); ++iter)
   {
     Glib::ustring strClausePart;
 
-    const Gnome::Gda::Value data = get_entered_field_data(*iter);
+    const Gnome::Gda::Value data = get_entered_field_data(*(*iter));
 
     if(!GlomConversions::value_is_empty(data))
     {
-      const Field field = iter->m_field;
+      const Field field = (*iter)->m_field;
 
       bool use_this_field = true;
       if(field.get_glom_type() == Field::TYPE_BOOLEAN) //TODO: We need an intermediate state for boolean fields, so that they can be ignored in searches.
@@ -82,8 +88,7 @@ Glib::ustring Box_Data::get_WhereClause() const
 
       if(use_this_field)
       {
-        //TODO: "table.name"
-        strClausePart = field.get_name() + " LIKE " +  field.sql(data); //% is mysql wildcard for 0 or more characters.
+        strClausePart = m_strTableName + "." + field.get_name() + " " + field.sql_find_operator() + " " +  field.sql_find(data); //% is mysql wildcard for 0 or more characters.
       }
     }
 
@@ -99,7 +104,16 @@ void Box_Data::on_Button_Find()
   //Make sure that the cell is updated:
   //m_AddDel.finish_editing();
 
-  signal_find.emit(get_WhereClause());
+  const Glib::ustring where_clause = get_find_where_clause();
+  if(where_clause.empty())
+  {
+    Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("No Find Criteria")), true, Gtk::MESSAGE_WARNING );
+    dialog.set_secondary_text(_("You have not entered any find criteria. Try entering information in the fields."));
+    dialog.set_transient_for(*get_app_window());
+    dialog.run();
+  }
+  else
+    signal_find_criteria.emit(where_clause);
 }
 
 Box_Data::type_map_fields Box_Data::get_record_field_values(const Gnome::Gda::Value& primary_key_value)
@@ -119,8 +133,8 @@ Box_Data::type_map_fields Box_Data::get_record_field_values(const Gnome::Gda::Va
     type_vecLayoutFields fieldsToGet;
     for(Document_Glom::type_vecFields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
     {
-      LayoutItem_Field layout_item;
-      layout_item.m_field = *iter;
+      sharedptr<LayoutItem_Field> layout_item(new LayoutItem_Field);
+      layout_item->m_field = *iter;
       fieldsToGet.push_back(layout_item);
     }
 
@@ -153,39 +167,44 @@ Glib::RefPtr<Gnome::Gda::DataModel> Box_Data::record_new(bool use_entered_data,
 
   const Glib::ustring primary_key_name = fieldPrimaryKey.get_name();
 
-  type_vecLayoutFields fieldsToAdd = m_Fields;
+  type_vecLayoutFields fieldsToAdd = m_FieldsShown;
 
-  //Add the primary key if it is not normally shown:
-  type_vecLayoutFields::const_iterator iterFindPrimary = std::find_if(fieldsToAdd.begin(), fieldsToAdd.end(), predicate_FieldHasName<LayoutItem_Field>(primary_key_name));
-  if(iterFindPrimary == fieldsToAdd.end())
+  //Add values for all fields, not just the shown ones:
+  //For instance, we must always add the primary key, and fields with default/calculated/lookup values:
+  for(type_vecFields::const_iterator iter = m_TableFields.begin(); iter != m_TableFields.end(); ++iter)
   {
-    LayoutItem_Field layout_item;
-    layout_item.m_field = fieldPrimaryKey;
+    //TODO: Search for the non-related field with the name, not just the field with the name:
+    type_vecLayoutFields::const_iterator iterFind = std::find_if(fieldsToAdd.begin(), fieldsToAdd.end(), predicate_FieldHasName<LayoutItem_Field>(iter->get_name()));
+    if(iterFind == fieldsToAdd.end())
+    {
+      sharedptr<LayoutItem_Field> layout_item(new LayoutItem_Field);
+      layout_item->m_field = *iter;
 
-    fieldsToAdd.push_back(layout_item);
+      fieldsToAdd.push_back(layout_item);
+    }
   }
 
   //Calculate any necessary field values and enter them:
-  for(type_vecLayoutFields::const_iterator iter = m_Fields.begin(); iter != m_Fields.end(); ++iter)
+  for(type_vecLayoutFields::const_iterator iter = fieldsToAdd.begin(); iter != fieldsToAdd.end(); ++iter)
   {
-    const LayoutItem_Field& layout_item = *iter;
+    sharedptr<LayoutItem_Field> layout_item = *iter;
 
     //If the user did not enter something in this field:
-    Gnome::Gda::Value value = get_entered_field_data(layout_item);
+    Gnome::Gda::Value value = get_entered_field_data(*layout_item);
 
     if(GlomConversions::value_is_empty(value)) //This deals with empty strings too.
     {
-      const Field& field = layout_item.m_field;
+      const Field& field = layout_item->m_field;
 
       //If the default value should be calculated, then calculate it:
-      const Glib::ustring calculation = field.get_calculation(); //TODO_Performance: Use a get_has_calculation() method.
-      if(!calculation.empty())
+      if(field.get_has_calculation())
       {
+        const Glib::ustring calculation = field.get_calculation();
         const type_map_fields field_values = get_record_field_values(primary_key_value);
 
         const Gnome::Gda::Value value = glom_evaluate_python_function_implementation(field.get_glom_type(), calculation, field_values,
           get_document(), m_strTableName);
-        set_entered_field_data(layout_item, value);
+        set_entered_field_data(*layout_item, value);
       }
 
       //Use default values (These are also specified in postgres as part of the field definition,
@@ -193,10 +212,10 @@ Glib::RefPtr<Gnome::Gda::DataModel> Box_Data::record_new(bool use_entered_data,
       //TODO_Performance: Add has_default_value()?
       if(GlomConversions::value_is_empty(value))
       {
-        const Gnome::Gda::Value default_value = field.get_field_info().get_default_value();
+        const Gnome::Gda::Value default_value = field.get_default_value();
         if(!GlomConversions::value_is_empty(default_value))
         {
-          set_entered_field_data(layout_item, default_value);
+          set_entered_field_data(*layout_item, default_value);
         }
       }
     }
@@ -212,16 +231,16 @@ Glib::RefPtr<Gnome::Gda::DataModel> Box_Data::record_new(bool use_entered_data,
 
   for(type_vecLayoutFields::const_iterator iter = fieldsToAdd.begin(); iter != fieldsToAdd.end(); ++iter)
   {
-    const LayoutItem_Field& layout_item = *iter;
-    const Glib::ustring field_name = layout_item.get_name();
-    if(!layout_item.get_has_relationship_name()) //TODO: Allow people to add a related record also by entering new data in a related field of the related record.
+    sharedptr<LayoutItem_Field> layout_item = *iter;
+    const Glib::ustring field_name = layout_item->get_name();
+    if(!layout_item->get_has_relationship_name()) //TODO: Allow people to add a related record also by entering new data in a related field of the related record.
     {
       type_map_added::const_iterator iterFind = map_added.find(field_name);
       if(iterFind == map_added.end()) //If it was not added already
       {
         Gnome::Gda::Value value;
 
-        const Field& field = layout_item.m_field;
+        const Field& field = layout_item->m_field;
 
         //Use the specified (generated) primary key value, if there is one:
         if(primary_key_name == field_name && !GlomConversions::value_is_empty(primary_key_value))
@@ -231,7 +250,7 @@ Glib::RefPtr<Gnome::Gda::DataModel> Box_Data::record_new(bool use_entered_data,
         else
         {
           if(use_entered_data || !field.get_calculation().empty()) //TODO_Performance: Use a get_has_calculation() method.
-            value = get_entered_field_data(layout_item);
+            value = get_entered_field_data(*layout_item);
         }
 
         Glib::ustring strFieldValue = field.sql(value);
@@ -273,11 +292,19 @@ bool Box_Data::get_unstored_data() const
   return m_bUnstoredData;
 }
 
-void Box_Data::fill_from_database()
+void Box_Data::create_layout()
 {
   set_unstored_data(false);
 
-  Box_DB_Table::fill_from_database();
+  //Cache the table information, for performance:
+  m_TableFields = get_fields_for_table(m_strTableName);
+}
+
+bool Box_Data::fill_from_database()
+{
+  set_unstored_data(false);
+
+  return Box_DB_Table::fill_from_database();
 }
 
 bool Box_Data::confirm_discard_unstored_data() const
@@ -287,7 +314,7 @@ bool Box_Data::confirm_discard_unstored_data() const
     //Ask user to confirm loss of data:
     Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("No primary key value")), true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_OK_CANCEL );
     dialog.set_secondary_text(_("This data can not be stored in the database because you have not provided a primary key.\nDo you really want to discard this data?"));
-    //TODO: dialog.set_transient_for(*this);
+    //TODO: It needs a const. I wonder if it should. murrayc. dialog.set_transient_for(*get_app_window());
     int iButton = dialog.run();
 
     return (iButton == Gtk::RESPONSE_OK);
@@ -302,7 +329,7 @@ void Box_Data::show_layout_dialog()
 {
   if(m_pDialogLayout)
   {
-    m_pDialogLayout->set_document(m_layout_name, get_document(), m_strTableName, m_Fields);
+    m_pDialogLayout->set_document(m_layout_name, get_document(), m_strTableName, m_FieldsShown); //TODO: Use m_TableFields?
     m_pDialogLayout->show();
   }
 }
@@ -310,7 +337,10 @@ void Box_Data::show_layout_dialog()
 void Box_Data::on_dialog_layout_hide()
 {
   //Re-fill view, in case the layout has changed:
-  fill_from_database();
+  create_layout();
+
+  if(ConnectionPool::get_instance()->get_ready_to_connect())
+    fill_from_database();
 }
 
 Box_Data::type_vecLayoutFields Box_Data::get_fields_to_show() const
@@ -361,7 +391,7 @@ void Box_Data::get_table_fields_to_show_add_group(const Glib::ustring& table_nam
             layout_item.m_priv_view = privs_related.m_view;
             layout_item.m_priv_edit = privs_related.m_edit;
 
-            vecFields.push_back(layout_item);
+            vecFields.push_back( sharedptr<LayoutItem_Field>(new LayoutItem_Field(layout_item)) );
           }
         }
       }
@@ -379,7 +409,7 @@ void Box_Data::get_table_fields_to_show_add_group(const Glib::ustring& table_nam
           layout_item.m_priv_view = table_privs.m_view;
           layout_item.m_priv_edit = table_privs.m_edit;
 
-          vecFields.push_back(layout_item);
+          vecFields.push_back( sharedptr<LayoutItem_Field>(new LayoutItem_Field(layout_item)) );
         }
       }
     }
@@ -434,17 +464,17 @@ Box_Data::type_vecLayoutFields Box_Data::get_table_fields_to_show(const Glib::us
       Glib::ustring primary_key_field_name;
       if(bPrimaryKeyFound)
       {
-        LayoutItem_Field layout_item;
-        layout_item.m_field = all_fields[iPrimaryKey];
+        sharedptr<LayoutItem_Field> layout_item(new LayoutItem_Field);
+        layout_item->m_field = all_fields[iPrimaryKey];
 
         //Don't use thousands separators with ID numbers:
-        layout_item.m_numeric_format.m_use_thousands_separator = false;
+        layout_item->m_formatting.m_numeric_format.m_use_thousands_separator = false;
 
-        layout_item.set_editable(true); //A sensible default.
+        layout_item->set_editable(true); //A sensible default.
 
         //Prevent editing of the field if the user may not edit this table:
-        layout_item.m_priv_view = table_privs.m_view;
-        layout_item.m_priv_edit = table_privs.m_edit;
+        layout_item->m_priv_view = table_privs.m_view;
+        layout_item->m_priv_edit = table_privs.m_edit;
 
         result.push_back(layout_item);
       }
@@ -456,14 +486,14 @@ Box_Data::type_vecLayoutFields Box_Data::get_table_fields_to_show(const Glib::us
 
         if(iter->get_name() != primary_key_field_name) //We already added the primary key.
         {
-          LayoutItem_Field layout_item;
-          layout_item.m_field = field_info;
+          sharedptr<LayoutItem_Field> layout_item(new LayoutItem_Field);
+          layout_item->m_field = field_info;
 
-          layout_item.set_editable(true); //A sensible default.
+          layout_item->set_editable(true); //A sensible default.
 
           //Prevent editing of the field if the user may not edit this table:
-          layout_item.m_priv_view = table_privs.m_view;
-          layout_item.m_priv_edit = table_privs.m_edit;
+          layout_item->m_priv_view = table_privs.m_view;
+          layout_item->m_priv_edit = table_privs.m_edit;
 
           result.push_back(layout_item);
         }
@@ -497,6 +527,11 @@ Box_Data::type_vecLayoutFields Box_Data::get_table_fields_to_show(const Glib::us
 
 Gnome::Gda::Value Box_Data::generate_next_auto_increment(const Glib::ustring& table_name, const Glib::ustring field_name)
 {
+  //Get it from the database system table.
+  //The developer can change the next value in the Database Preferences
+  return get_next_auto_increment_value(table_name, field_name);
+
+  /*
   //This is a workaround for postgres problems. Ideally, we need to use the postgres serial type and find out how to get the generated value after we add a row.
 
   double result = 0;
@@ -520,9 +555,10 @@ Gnome::Gda::Value Box_Data::generate_next_auto_increment(const Glib::ustring& ta
   //Get a string representation of the number, so we can put it back in a NUMERIC Gda::Value:
 
   return GlomConversions::parse_value(result);
+  */
 }
 
-/** Get the fields that are in related tables, via a relationship using @a field_name changes.
+/** Get the shown fields that are in related tables, via a relationship using @a field_name changes.
  */
 Box_Data::type_vecLayoutFields Box_Data::get_related_fields(const Glib::ustring& field_name) const
 {
@@ -531,15 +567,15 @@ Box_Data::type_vecLayoutFields Box_Data::get_related_fields(const Glib::ustring&
   const Document_Glom* document = dynamic_cast<const Document_Glom*>(get_document());
   if(document)
   {
-    for(type_vecLayoutFields::const_iterator iter = m_Fields.begin(); iter != m_Fields.end();  ++iter)
+    for(type_vecLayoutFields::const_iterator iter = m_FieldsShown.begin(); iter != m_FieldsShown.end();  ++iter)
     {
-      const LayoutItem_Field& layout_field = *iter;
+      sharedptr<LayoutItem_Field> layout_field = *iter;
       //Examine each field that looks up its data from a relationship:
-      if(layout_field.get_has_relationship_name())
+      if(layout_field->get_has_relationship_name())
       {
         //Get the relationship information:
         Relationship relationship;
-        bool test = document->get_relationship(m_strTableName, layout_field.get_relationship_name(), relationship);
+        bool test = document->get_relationship(m_strTableName, layout_field->get_relationship_name(), relationship);
         if(test)
         {
           //If the relationship uses the specified field:
@@ -556,23 +592,28 @@ Box_Data::type_vecLayoutFields Box_Data::get_related_fields(const Glib::ustring&
   return result;
 }
 
-Box_Data::type_vecLayoutFields Box_Data::get_calculated_fields(const Glib::ustring& field_name) const
+Box_Data::type_field_calcs Box_Data::get_calculated_fields(const Glib::ustring& field_name)
 {
-  type_vecLayoutFields result;
+  type_field_calcs result;
 
   const Document_Glom* document = dynamic_cast<const Document_Glom*>(get_document());
   if(document)
   {
-    //TODO: examine all fields, not just the the shown fields (m_Fields):
-    for(type_vecLayoutFields::const_iterator iter = m_Fields.begin(); iter != m_Fields.end();  ++iter)
+    //Examine all fields, not just the the shown fields.
+    for(type_vecFields::const_iterator iter = m_TableFields.begin(); iter != m_TableFields.end();  ++iter)
     {
-      const Field& field = iter->m_field;
+      const Field& field = *iter;
 
       //Does this field's calculation use the field?
       const Field::type_list_strings fields_triggered = field.get_calculation_fields();
       Field::type_list_strings::const_iterator iterFind = std::find(fields_triggered.begin(), fields_triggered.end(), field_name);
       if(iterFind != fields_triggered.end())
-        result.push_back(*iter);
+      {
+        CalcInProgress item;
+        item.m_field = field;
+
+        result[field.get_name()] = item;
+      }
     }
   }
 
@@ -589,10 +630,11 @@ Box_Data::type_list_lookups Box_Data::get_lookup_fields(const Glib::ustring& fie
   const Document_Glom* document = dynamic_cast<const Document_Glom*>(get_document());
   if(document)
   {
-    //TODO: examine all fields, not just the the shown fields (m_Fields):
-    for(type_vecLayoutFields::const_iterator iter = m_Fields.begin(); iter != m_Fields.end();  ++iter)
+    //Examine all fields, not just the the shown fields (m_Fields):
+    for(type_vecFields::const_iterator iter = m_TableFields.begin(); iter != m_TableFields.end();  ++iter)
     {
-      const Field& field = iter->m_field;
+      const Field& field = *iter;
+
       //Examine each field that looks up its data from a relationship:
       if(field.get_is_lookup())
       {
@@ -606,7 +648,6 @@ Box_Data::type_list_lookups Box_Data::get_lookup_fields(const Glib::ustring& fie
           {
             //Add it:
             LayoutItem_Field item;
-            item.set_name(field.get_name());
             item.m_field = field;
             result.push_back( type_pairFieldTrigger(item, relationship) );
           }
@@ -644,49 +685,138 @@ Gnome::Gda::Value Box_Data::get_lookup_value(const Relationship& relationship, c
   return result;
 }
 
-void Box_Data::do_calculations(const LayoutItem_Field& field_changed, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
+void Box_Data::calculate_field(const Field& field, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
 {
-  //TODO: Get (recursively) the list of fields to recalcualte, sort them in order of dependency, then calculate them all.
+  const Glib::ustring field_name = field.get_name();
+  //g_warning("Box_Data::calculate_field(): field_name=%s", field_name.c_str());
 
-  //g_warning("Box_Data::do_calculations(): field_changed=%s", field_changed.get_name().c_str());
-
-  //TODO_Performance: This is profoundly inefficient:
-
-  //Recalculate fields that are triggered by a change of this field's value:
-  const type_vecLayoutFields calculated_fields = get_calculated_fields(field_changed.get_name());
-  for(type_vecLayoutFields::const_iterator iter = calculated_fields.begin(); iter != calculated_fields.end(); ++iter)
+  //Do we already have this in our list?
+  type_field_calcs::iterator iterFind = m_FieldsCalculationInProgress.find(field_name);
+  if(iterFind == m_FieldsCalculationInProgress.end()) //If it was not found.
   {
-    const LayoutItem_Field& field = *iter;
+    //Add it:
+    CalcInProgress item;
+    item.m_field = field;
+    m_FieldsCalculationInProgress[field_name] = item;
 
-    type_vecLayoutFields::const_iterator iterFind = std::find(m_FieldsCalculationInProgress.begin(), m_FieldsCalculationInProgress.end(), field);
-    if(iterFind != m_FieldsCalculationInProgress.end())
+    iterFind = m_FieldsCalculationInProgress.find(field_name); //Always succeeds.
+  }
+
+  CalcInProgress& refCalcProgress = iterFind->second;
+
+  //Use the previously-calculated value if possible:
+  if(refCalcProgress.m_calc_in_progress)
+  {
+    //g_warning("  Box_Data::calculate_field(): Circular calculation detected. field_name=%s", field_name.c_str());
+    //refCalcProgress.m_value = GlomConversions::get_empty_value(field.get_glom_type()); //Give up.
+  }
+  else if(refCalcProgress.m_calc_finished)
+  {
+    //g_warning("  Box_Data::calculate_field(): Already calculated.");
+
+    //Don't bother calculating it again. The correct value is already in the database and layout.
+  }
+  else
+  {
+    //g_warning("  Box_Data::calculate_field(): setting calc_in_progress: field_name=%s", field_name.c_str());
+
+    refCalcProgress.m_calc_in_progress = true; //Let the recursive calls to calculate_field() check this.
+
+    LayoutItem_Field layout_item;
+    layout_item.m_field = refCalcProgress.m_field;
+
+    //Calculate dependencies first:
+    const Field::type_list_strings fields_needed = field.get_calculation_fields();
+    for(Field::type_list_strings::const_iterator iterNeeded = fields_needed.begin(); iterNeeded != fields_needed.end(); ++iterNeeded)
     {
-      g_warning("Box_Data::do_calculations(): Circular calculation detected. The Infinite loop was prevented before re-calculating field %s", field.get_name().c_str());
+      Field field_needed;
+      //g_warning("field %s needs %s", field_name.c_str(), iterNeeded->c_str());
+      bool test = get_document()->get_field(m_strTableName, *iterNeeded, field_needed);
+      if(test)
+      {
+        if(field_needed.get_has_calculation())
+        {
+          //g_warning("  calling calculate_field() for %s", iterNeeded->c_str());
+          calculate_field(field_needed, primary_key, primary_key_value);
+        }
+        else
+        {
+          //g_warning("  not a calculated field.");
+        }
+      }
+    }
+
+
+    //m_FieldsCalculationInProgress has changed, probably invalidating our iter, so get it again:
+    iterFind = m_FieldsCalculationInProgress.find(field_name); //Always succeeds.
+    CalcInProgress& refCalcProgress = iterFind->second;
+
+    //Check again, because the value miight have been calculated during the dependencies.
+    if(refCalcProgress.m_calc_finished)
+    {
+      //We recently calculated this value, and set it in the database and layout, so don't waste time doing it again:
     }
     else
     {
-      //g_warning("CircularPrevention: recalculating field=%s", field.get_name().c_str());
-
       //recalculate:
       const type_map_fields field_values = get_record_field_values(primary_key_value);
-      Gnome::Gda::Value value = glom_evaluate_python_function_implementation(field.m_field.get_glom_type(), iter->m_field.get_calculation(), field_values,
+      refCalcProgress.m_value  = glom_evaluate_python_function_implementation(refCalcProgress.m_field.get_glom_type(), refCalcProgress.m_field.get_calculation(), field_values,
             get_document(), m_strTableName);
 
+      refCalcProgress.m_calc_finished = true;
+      refCalcProgress.m_calc_in_progress = false;
+
+      LayoutItem_Field layout_item;
+      layout_item.m_field = refCalcProgress.m_field;
+ 
       //show it:
-      set_entered_field_data(*iter, value);
+      set_entered_field_data(layout_item, refCalcProgress.m_value );
 
       //Add it to the database (even if it is not shown in the view)
-      set_field_value_in_database(*iter, value, primary_key, primary_key_value); //This triggers other recalculations/lookups.
+      //Using true for the last parameter means we use existing calculations where possible,
+      //instead of recalculating a field that is being calculated already, and for which this dependent field is being calculated anyway.
+      set_field_value_in_database(layout_item, refCalcProgress.m_value, primary_key, primary_key_value, true); //This triggers other recalculations/lookups.
     }
   }
+
 }
 
-bool Box_Data::set_field_value_in_database(const LayoutItem_Field& field_layout, const Gnome::Gda::Value& field_value, const Field& primary_key_field, const Gnome::Gda::Value& primary_key_value)
+
+void Box_Data::do_calculations(const LayoutItem_Field& field_changed, const Field& primary_key, const Gnome::Gda::Value& primary_key_value, bool first_calc_field)
 {
-  return set_field_value_in_database(Gtk::TreeModel::iterator(), field_layout, field_value, primary_key_field, primary_key_value);
+  //g_warning("Box_Data::do_calculations(): triggered by =%s", field_changed.get_name().c_str());
+
+  if(first_calc_field)
+  {
+    //g_warning("  clearing m_FieldsCalculationInProgress");
+    m_FieldsCalculationInProgress.clear();
+  }
+
+  //Recalculate fields that are triggered by a change of this field's value, not including calculations that these calculations use.
+  type_field_calcs calculated_fields = get_calculated_fields(field_changed.get_name()); //TODO: Just return the Fields, not the CalcInProgress.
+  for(type_field_calcs::iterator iter = calculated_fields.begin(); iter != calculated_fields.end(); ++iter)
+  {
+    const Field& field = iter->second.m_field;
+
+    calculate_field(field, primary_key, primary_key_value); //And any dependencies.
+
+    //Calculate anything that depends on this.
+    LayoutItem_Field layout_item;
+    layout_item.m_field = field;
+
+    do_calculations(layout_item, primary_key, primary_key_value, false /* recurse, reusing m_FieldsCalculationInProgress */);
+  }
+
+  if(first_calc_field)
+    m_FieldsCalculationInProgress.clear();
 }
 
-bool Box_Data::set_field_value_in_database(const Gtk::TreeModel::iterator& row, const LayoutItem_Field& field_layout, const Gnome::Gda::Value& field_value, const Field& primary_key_field, const Gnome::Gda::Value& primary_key_value)
+bool Box_Data::set_field_value_in_database(const LayoutItem_Field& field_layout, const Gnome::Gda::Value& field_value, const Field& primary_key_field, const Gnome::Gda::Value& primary_key_value, bool use_current_calculations)
+{
+  return set_field_value_in_database(Gtk::TreeModel::iterator(), field_layout, field_value, primary_key_field, primary_key_value, use_current_calculations);
+}
+
+bool Box_Data::set_field_value_in_database(const Gtk::TreeModel::iterator& row, const LayoutItem_Field& field_layout, const Gnome::Gda::Value& field_value, const Field& primary_key_field, const Gnome::Gda::Value& primary_key_value, bool use_current_calculations)
 {
   //row is invalid, and ignored, for Box_Data_Details.
 
@@ -695,7 +825,22 @@ bool Box_Data::set_field_value_in_database(const Gtk::TreeModel::iterator& row,
   const Glib::ustring field_name = field_layout.get_name();
   if(!field_name.empty()) //This should not happen.
   {
-    Glib::ustring strQuery = "UPDATE " + m_strTableName;
+    Glib::ustring table_name;
+    if(field_layout.get_has_relationship_name())
+    {
+      //The field is in a related table.
+      table_name = field_layout.m_relationship.get_to_table();
+    }
+    else
+      table_name = m_strTableName;
+      
+    if(table_name.empty())
+    {
+      g_warning("Box_Data::set_field_value_in_database(): table_name is empty.");
+      return false;
+    }
+    
+    Glib::ustring strQuery = "UPDATE " + table_name;
     strQuery += " SET " + field_name + " = " + field.sql(field_value);
     strQuery += " WHERE " + primary_key_field.get_name() + " = " + primary_key_field.sql(primary_key_value);
     Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute(strQuery);  //TODO: Handle errors
@@ -715,16 +860,9 @@ bool Box_Data::set_field_value_in_database(const Gtk::TreeModel::iterator& row,
     //TODO: Make lookups part of this?
     //Prevent circular calculations during the recursive do_calculations:
     {
-      m_FieldsCalculationInProgress.push_back(field_layout);
-
       //Recalculate any calculated fields that depend on this calculated field.
-      do_calculations(field_layout, primary_key_field, primary_key_value);
-
-      type_vecLayoutFields::iterator iterFind = std::find(m_FieldsCalculationInProgress.begin(), m_FieldsCalculationInProgress.end(), field_layout);
-      if(iterFind != m_FieldsCalculationInProgress.end())
-      {
-        m_FieldsCalculationInProgress.erase(iterFind);
-      }
+      //g_warning("Box_Data::set_field_value_in_database(): calling do_calculations");
+      do_calculations(field_layout, primary_key_field, primary_key_value, !use_current_calculations);
     }
   }
 
@@ -826,7 +964,7 @@ bool Box_Data::get_field_primary_key_index(const type_vecFields& fields, guint&
   guint cols_count = fields.size();
   while(col < cols_count)
   {
-    if(fields[col].get_field_info().get_primary_key())
+    if(fields[col].get_primary_key())
     {
       field_column = col;
       return true;
@@ -851,7 +989,7 @@ bool Box_Data::get_field_primary_key_index(const type_vecLayoutFields& fields, g
   guint cols_count = fields.size();
   while(col < cols_count)
   {
-    if(fields[col].m_field.get_field_info().get_primary_key())
+    if(fields[col]->m_field.get_primary_key())
     {
       field_column = col;
       return true;
@@ -880,10 +1018,12 @@ bool Box_Data::record_delete(const Gnome::Gda::Value& primary_key_value)
   }
 }
 
+/*
 bool Box_Data::get_field(const Glib::ustring& name, Field& field) const
 {
-  type_vecLayoutFields::const_iterator iterFind = std::find_if( m_Fields.begin(), m_Fields.end(), predicate_FieldHasName<LayoutItem_Field>(name) );
-  if(iterFind != m_Fields.end()) //If it was found:
+g_warning("debug: Box_Data::get_field()");
+  type_vecFields::const_iterator iterFind = std::find_if( m_TableFields.begin(), m_TableFields.end(), predicate_FieldHasName<LayoutItem_Field>(name) );
+  if(iterFind != m_FieldsShown.end()) //If it was found:
   {
     field = iterFind->m_field;
     return true;
@@ -893,13 +1033,14 @@ bool Box_Data::get_field(const Glib::ustring& name, Field& field) const
     return false; //not found.
   }
 }
+*/
 
 Glib::ustring Box_Data::build_sql_select(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const Field& primary_key_field, const Gnome::Gda::Value& primary_key_value)
 {
   if(!GlomConversions::value_is_empty(primary_key_value)) //If there is a record to show:
   {
     const Glib::ustring where_clause = m_strTableName + "." + primary_key_field.get_name() + " = " + primary_key_field.sql(primary_key_value);
-    return util_build_sql_select_with_where_clause(table_name, fieldsToGet, where_clause);
+    return GlomUtils::build_sql_select_with_where_clause(table_name, fieldsToGet, where_clause);
   }
 
   return Glib::ustring();
@@ -970,23 +1111,6 @@ Glib::ustring Box_Data::build_sql_select_with_where_clause(const Glib::ustring&
 }
 */
 
-Glib::ustring Box_Data::get_layout_item_table_name(const LayoutItem_Field& layout_item, const Glib::ustring table_name)
-{
-  if(!layout_item.get_has_relationship_name())
-    return table_name;
-  else
-  {
-    const Glib::ustring relationship_name = layout_item.get_relationship_name();
-    Relationship relationship;
-    Document_Glom* document = get_document();
-    bool test = document->get_relationship(table_name, relationship_name, relationship);
-    if(test)
-     return relationship.get_to_table();
-  }
-
-  return Glib::ustring();
-}
-
 bool Box_Data::get_related_record_exists(const Relationship& relationship, const Field& key_field, const Gnome::Gda::Value& key_value)
 {
   bool result = false;
@@ -1038,7 +1162,7 @@ bool Box_Data::add_related_record_for_field(const LayoutItem_Field& layout_item_
     }
     else
     {
-      const bool key_is_auto_increment = primary_key_field.get_field_info().get_auto_increment();
+      const bool key_is_auto_increment = primary_key_field.get_auto_increment();
 
       //If we would have to set an otherwise auto-increment key to add the record.
       if( key_is_auto_increment && !GlomConversions::value_is_empty(primary_key_value) )
@@ -1089,15 +1213,25 @@ bool Box_Data::add_related_record_for_field(const LayoutItem_Field& layout_item_
               Field parent_primary_key_field;
               bool test = get_field_primary_key(parent_primary_key_field);
               if(!test)
+              {
+                g_warning("Box_Data::add_related_record_for_field(): get_field_primary_key() failed. table = %s", get_table_name().c_str());
                 return false;
+              }
               else
               {
                 const Gnome::Gda::Value parent_primary_key_value = get_primary_key_value_selected();
-
-                const Glib::ustring strQuery = "UPDATE  " + relationship.get_from_table() + " SET " + relationship.get_from_field() + " = " + primary_key_field.sql(primary_key_value) +
-                  " WHERE " + relationship.get_from_table() + "." + parent_primary_key_field.get_name() + " = " + parent_primary_key_field.sql(parent_primary_key_value);
-                bool test = Query_execute(strQuery);
-                return test;
+                if(parent_primary_key_value.is_null())
+                {
+                  g_warning("Box_Data::add_related_record_for_field(): get_primary_key_value_selected() failed. table = %s", get_table_name().c_str());
+                  return false;
+                }
+                else
+                {
+                  const Glib::ustring strQuery = "UPDATE  " + relationship.get_from_table() + " SET " + relationship.get_from_field() + " = " + primary_key_field.sql(primary_key_value) +
+                    " WHERE " + relationship.get_from_table() + "." + parent_primary_key_field.get_name() + " = " + parent_primary_key_field.sql(parent_primary_key_value);
+                  bool test = Query_execute(strQuery);
+                  return test;
+                }
               }
             }
           }
@@ -1118,44 +1252,20 @@ void Box_Data::print_layout()
   dialog.run();
 }
 
-Glib::ustring Box_Data::xslt_process(const xmlpp::Document& xml_document, const std::string& filepath_xslt)
+Glib::ustring Box_Data::get_layout_name() const
 {
-  Glib::ustring  result;
-
-  //Use libxslt to transform the XML:
-  xmlDocPtr style = xmlReadFile(filepath_xslt.c_str(), 0, 0);
-  if(style)
-  {
-    //Parse the stylesheet:
-    xsltStylesheetPtr cur = xsltParseStylesheetDoc(style);
-    if(cur)
-    {
-      //Use the parsed stylesheet on the XML:
-      xmlDocPtr pDocOutput = xsltApplyStylesheet(cur, const_cast<xmlDoc*>(xml_document.cobj()), 0);
-      xsltFreeStylesheet(cur);
-
-      //Get the output text:
-      xmlChar* buffer = 0;
-      int length = 0;
-      xmlDocDumpFormatMemoryEnc(pDocOutput, &buffer, &length, 0, 0);
-
-      if(buffer)
-      {
-        // Create a Glib::ustring copy of the buffer
-
-        // Here we force the use of Glib::ustring::ustring( InputIterator begin, InputIterator end )
-        // instead of Glib::ustring::ustring( const char*, size_type ) because it
-        // expects the length of the string in characters, not in bytes.
-        result = Glib::ustring( reinterpret_cast<const char *>(buffer), reinterpret_cast<const char *>(buffer + length) );
-
-        // Deletes the original buffer
-        xmlFree(buffer);
-      }
-
-      xmlFreeDoc(pDocOutput);
-    }
-  }
-
-  return result;
+  return m_layout_name;
 }
 
+bool Box_Data::confirm_delete_record()
+{
+  //Ask the user for confirmation:
+  Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Delete record")), true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE);
+  dialog.set_secondary_text(_("Are you sure that you would like to delete this record? The data in this record will then be permanently lost."));
+  dialog.set_transient_for(*get_app_window());
+  dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+  dialog.add_button(Gtk::Stock::DELETE, Gtk::RESPONSE_OK);
+
+  const int response = dialog.run();
+  return (response == Gtk::RESPONSE_OK);
+}
