@@ -39,16 +39,14 @@ public:
   virtual ~Box_Data();
 
   ///Create the layout for the database structure, and fill it with data from the database.
-  virtual void init_db_details(const Glib::ustring& strTableName, const Glib::ustring& strWhereClause = Glib::ustring());
+  virtual bool init_db_details(const Glib::ustring& strTableName, const Glib::ustring& strWhereClause = Glib::ustring());
 
   //Fill the existing layout with data from the databse.
-  virtual void refresh_data_from_database(const Glib::ustring& strWhereClause = Glib::ustring());
+  virtual bool refresh_data_from_database(const Glib::ustring& strWhereClause = Glib::ustring());
 
   virtual void print_layout(); //A test, for now.
 
-  typedef std::vector<LayoutItem_Field> type_vecLayoutFields;
-
-  virtual Glib::ustring get_WhereClause() const;
+  virtual Glib::ustring get_find_where_clause() const;
 
   virtual void set_unstored_data(bool bVal);
   virtual bool get_unstored_data() const;
@@ -56,13 +54,19 @@ public:
   virtual bool confirm_discard_unstored_data() const;
 
   virtual void show_layout_dialog();
+  
+  Glib::ustring get_layout_name() const;
 
   //Signals:
 
-  //signal_find: Used by _Find sub-classes.
+  /** Emitted when the user has entered a find critera that
+   * should be used to find and display records.
+   * Used by _Find sub-classes.
+   * @param find_criteria The SQL where clause.
+   */
   //Should be a MI class, derived by those sub-classes. TODO.
   //where_clause.
-  sigc::signal<void, Glib::ustring> signal_find;
+  sigc::signal<void, Glib::ustring> signal_find_criteria;
 
   //g++ 3.4 needs this to be public when used from Box_Data_Details. I'm not sure why. murrayc.
   virtual void on_dialog_layout_hide();
@@ -75,7 +79,7 @@ protected:
   virtual void create_layout();
 
   ///Fill the existing layout with data from the database.
-  virtual void fill_from_database(); //override.
+  virtual bool fill_from_database(); //override.
 
   virtual void do_lookups(const Gtk::TreeModel::iterator& row, const LayoutItem_Field& field_changed, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value) = 0;
   virtual void refresh_related_fields(const Gtk::TreeModel::iterator& row, const LayoutItem_Field& field_changed, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value) = 0;
@@ -141,9 +145,8 @@ protected:
   virtual Gnome::Gda::Value get_primary_key_value_selected() = 0;
   //virtual bool get_field(const Glib::ustring& name, Field& field) const;
 
-  ///Get the table name. It's either the current table or the relationship's to_table:
-  Glib::ustring get_layout_item_table_name(const LayoutItem_Field& layout_item, const Glib::ustring& table_name);
-
+  bool confirm_delete_record();
+  
   //Signal handlers:
   virtual void on_Button_Find(); //only used by _Find sub-classes. Should be MI.
 
