@@ -182,6 +182,10 @@ bool Box_Data_Details::fill_from_database()
 
   Bakery::BusyCursor(*get_app_window());
 
+  const bool primary_key_is_empty = GlomConversions::value_is_empty(m_primary_key_value);
+  if(!primary_key_is_empty)
+    get_document()->set_layout_record_viewed(m_strTableName, m_layout_name, m_primary_key_value);
+    
   try
   {
 
@@ -215,9 +219,12 @@ bool Box_Data_Details::fill_from_database()
         const int index_primary_key = fieldsToGet.size() - 1;
 
         const Glib::ustring query = build_sql_select(m_strTableName, fieldsToGet, m_field_primary_key, m_primary_key_value);
-        Glib::RefPtr<Gnome::Gda::DataModel> result = Query_execute(query);
+        Glib::RefPtr<Gnome::Gda::DataModel> result;
+         
+        if(!primary_key_is_empty)
+          result = Query_execute(query);
 
-        if(result && result->get_n_rows())
+        if((result && result->get_n_rows()) || primary_key_is_empty) //either a working result or no result needed.
         {
           const Document_Glom* pDoc = dynamic_cast<const Document_Glom*>(get_document());
           if(pDoc)
@@ -226,11 +233,18 @@ bool Box_Data_Details::fill_from_database()
             //Document_Glom::type_vecFields vecFields = pDoc->get_table_fields(m_strTableName);
 
             const int row_number = 0; //The only row.
-            const int cols_count = result->get_n_columns();
+            int cols_count = 0;
+            if(!primary_key_is_empty)
+              cols_count = result->get_n_columns();
+            else
+              cols_count = fieldsToGet.size();
 
             //Get special possibly-non-visible field values:
-            if(index_primary_key < cols_count)
-              m_primary_key_value = result->get_value_at(index_primary_key, row_number);
+            if(!primary_key_is_empty)
+            {
+              if(index_primary_key < cols_count)
+                m_primary_key_value = result->get_value_at(index_primary_key, row_number);
+            }
 
             //Get field values to show:
             for(int i = 0; i < cols_count; ++i)
@@ -238,7 +252,15 @@ bool Box_Data_Details::fill_from_database()
               sharedptr<LayoutItem_Field> layout_item = fieldsToGet[i];
 
               //Field value:
-              Gnome::Gda::Value value = result->get_value_at(i, row_number);
+              Gnome::Gda::Value value;
+               
+              if(!primary_key_is_empty)
+                value = result->get_value_at(i, row_number);
+              else
+              {
+                value = GlomConversions::get_empty_value(layout_item->m_field.get_glom_type());
+              }
+               
               m_FlowTable.set_field_value(*layout_item, value);
             }
           }
@@ -300,13 +322,16 @@ void Box_Data_Details::on_button_del()
   }
   else
   {
-    bool bTest = record_delete(m_primary_key_value);
-
-    if(bTest)
+    if(confirm_delete_record())
     {
-      //Tell the list that it has been deleted:
-      //It will go to the next (or last) record,
-      signal_record_deleted().emit(m_primary_key_value);
+      bool bTest = record_delete(m_primary_key_value);
+  
+      if(bTest)
+      {
+        //Tell the list that it has been deleted:
+        //It will go to the next (or last) record,
+        signal_record_deleted().emit(m_primary_key_value);
+      }
     }
   }
 }
@@ -492,7 +517,7 @@ void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_
   const Glib::ustring strFieldName = layout_field.get_name();
 
   Gnome::Gda::Value primary_key_value = get_primary_key_value();
-  if(!GlomConversions::value_is_empty(primary_key_value)) //If there is a primary key value:
+  if(!GlomConversions::value_is_empty(primary_key_value)) //If there is not a primary key value:
   {
     Glib::ustring table_name;
     Field primary_key_field;
@@ -598,7 +623,7 @@ void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_
   else
   {
     //There is no current primary key value:
-
+    
     if(m_field_primary_key.get_auto_increment()) //If the primary key is an auto-increment:
     {
       if(strFieldName == m_field_primary_key.get_name()) //If edited field is the primary key.
@@ -609,6 +634,14 @@ void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_
         dialog.set_transient_for(*get_app_window());
         dialog.run();
       }
+      else
+      {
+        //Make a new record, and show it:
+        Gnome::Gda::Value primary_key_value = generate_next_auto_increment(m_strTableName, m_field_primary_key.get_name());
+
+        record_new(true /* use entered field data */, primary_key_value);
+        refresh_data_from_database(primary_key_value);
+      }
     }
     else
     {
