@@ -32,14 +32,20 @@ public:
 
   virtual Gnome::Gda::Value get_primary_key_value(const Gtk::TreeModel::iterator& row);
   virtual Gnome::Gda::Value get_primary_key_value_selected();
+  Gnome::Gda::Value get_primary_key_value_first();
 
   virtual Gnome::Gda::Value get_entered_field_data(const LayoutItem_Field& field) const;
   virtual void set_entered_field_data(const LayoutItem_Field& field, const Gnome::Gda::Value& value);
 
   virtual guint get_records_count() const;
+  
+  void set_read_only(bool read_only = true);
+  
+  //For instance, change "Open" to "Select" when used to select an ID.
+  void set_open_button_title(const Glib::ustring& title);
 
   //Primary Key value:
-  typedef sigc::signal<void, Gnome::Gda::Value> type_signal_user_requested_details;
+  typedef sigc::signal<void, const Gnome::Gda::Value&> type_signal_user_requested_details;
   type_signal_user_requested_details signal_user_requested_details();
 
   //Signal Handlers:
@@ -47,7 +53,7 @@ public:
   virtual void on_details_nav_previous();
   virtual void on_details_nav_next();
   virtual void on_details_nav_last();
-  virtual void on_Details_record_deleted(Gnome::Gda::Value primary_key_value);
+  virtual void on_Details_record_deleted(const Gnome::Gda::Value& primary_key_value);
 
 protected:
   virtual void create_layout(); //override
@@ -78,11 +84,13 @@ protected:
 
   virtual void print_layout();
   virtual void print_layout_group(xmlpp::Element* node_parent, const LayoutGroup& group);
+  
 
   //Member widgers:
   mutable DbAddDel_WithButtons m_AddDel; //mutable because its get_ methods aren't const.
 
   bool m_has_one_or_more_records;
+  bool m_read_only;
 
   type_signal_user_requested_details m_signal_user_requested_details;
 };
