@@ -20,8 +20,8 @@
 
 #include "adddel.h"
 #include <algorithm> //For std::find.
-#include <libintl.h>
-#include "cellrendererlist.h"
+#include <glibmm/i18n.h>
+#include "../cellrendererlist.h"
 #include "treeviewcolumn_glom.h"
 #include "../../data_structure/glomconversions.h"
 #include "../../dialog_invalid_data.h"
@@ -63,12 +63,30 @@ AddDelColumnInfo& AddDelColumnInfo::operator=(const AddDelColumnInfo& src)
 }
 
 AddDel::AddDel()
-:  m_col_key(0),
+: m_col_key(0),
   m_pColumnHeaderPopup(0),
   m_allow_column_chooser(false),
   m_auto_add(true),
   m_allow_add(true),
   m_allow_delete(true)
+{
+  init();
+}
+
+
+AddDel::AddDel(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& /* refGlade */)
+: Gtk::VBox(cobject),
+  m_col_key(0),
+  m_pColumnHeaderPopup(0),
+  m_allow_column_chooser(false),
+  m_auto_add(true),
+  m_allow_add(true),
+  m_allow_delete(true)
+{
+  init();
+}
+
+void AddDel::init()
 {
   set_prevent_user_signals();
   set_ignore_treeview_signals();
@@ -77,13 +95,13 @@ AddDel::AddDel()
 
   m_bAllowUserActions = true;
 
-  m_strSelectText = gettext("Edit");
-
   //Start with a useful default TreeModel:
   //set_columns_count(1);
   //construct_specified_columns();
 
   m_ScrolledWindow.add(m_TreeView);
+  m_ScrolledWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
+  m_ScrolledWindow.set_shadow_type(Gtk::SHADOW_IN);
   m_TreeView.show();
   pack_start(m_ScrolledWindow);
 
@@ -107,10 +125,11 @@ AddDel::AddDel()
   set_ignore_treeview_signals(false);
 
   remove_all_columns(); //set up the default columns.
-  
+
   show_all_children();
 }
 
+
 AddDel::~AddDel()
 {
 }
@@ -126,7 +145,7 @@ AddDel::on_MenuPopup_activate_Edit()
     {
       //Discover whether it's the last (empty) row:
       if(get_is_placeholder_row(iter))
-      {        
+      {
         //This is a new entry:
         signal_user_added()(iter);
 
@@ -183,20 +202,20 @@ void AddDel::setup_menu()
   m_MenuPopup.items().clear();
 
   //Add new menu items:
-  Gtk::Menu_Helpers::MenuElem menuItem_Edit(m_strSelectText);
+  Gtk::Menu_Helpers::MenuElem menuItem_Edit(_("Edit"));
   m_MenuPopup.items().push_back(menuItem_Edit);
   m_MenuPopup.items().back().signal_activate().connect(sigc::mem_fun(*this, &AddDel::on_MenuPopup_activate_Edit));
 
   if(get_allow_user_actions())
   {
-    Gtk::Menu_Helpers::MenuElem menuItem_Delete(gettext("Delete"));
+    Gtk::Menu_Helpers::MenuElem menuItem_Delete(_("Delete"));
     m_MenuPopup.items().push_back(menuItem_Delete);
     m_MenuPopup.items().back().signal_activate().connect(sigc::mem_fun(*this, &AddDel::on_MenuPopup_activate_Delete));
   }
   
   if(m_allow_column_chooser)
   {
-    Gtk::Menu_Helpers::MenuElem menuitem_ChooseColumns(gettext("Choose columns"));
+    Gtk::Menu_Helpers::MenuElem menuitem_ChooseColumns(_("Choose columns"));
     m_MenuPopup.items().push_back(menuitem_ChooseColumns);
     m_MenuPopup.items().back().signal_activate().connect(sigc::mem_fun(*this, &AddDel::on_MenuPopup_activate_ChooseColumns));
   }
@@ -285,12 +304,6 @@ void AddDel::remove_all()
 }
 
 
-Gnome::Gda::Value AddDel::get_value_as_value(const Gtk::TreeModel::iterator& iter, guint col)
-{
-  bool success = false;
-  return GlomConversions::parse_value(m_ColumnTypes[col].m_field_type, get_value(iter, col), success );
-}
-
 Glib::ustring AddDel::get_value(const Gtk::TreeModel::iterator& iter, guint col)
 {
   Glib::ustring value;
@@ -318,10 +331,6 @@ Glib::ustring AddDel::get_value(const Gtk::TreeModel::iterator& iter, guint col)
     }
   }
 
-  //TODO: This is a hack. We need to deal with NULL explicitly:
-  if(value == "NULL")
-    value = "";
-
   return value;
 }
 
@@ -337,22 +346,12 @@ Glib::ustring AddDel::get_value_key_selected()
   return get_value_selected(m_col_key);
 }
 
-Gnome::Gda::Value AddDel::get_value_key_selected_as_value()
-{
-  bool parsed = false;
-  return GlomConversions::parse_value(m_key_field.get_glom_type(), get_value_key_selected(), parsed);
-}
-
 Glib::ustring AddDel::get_value_selected(guint col)
 {
   Glib::ustring strValue = get_value(get_item_selected(), col);
   return strValue;
 }
 
-Gnome::Gda::Value AddDel::get_value_selected_as_value(guint col)
-{
-  return get_value_as_value(get_item_selected(), col);
-}
 
 Gtk::TreeModel::iterator AddDel::get_item_selected()
 {
@@ -390,7 +389,7 @@ bool AddDel::select_item(const Gtk::TreeModel::iterator& iter, guint column, boo
 {
   if(!m_refListStore)
     return false;
-   
+
   InnerIgnore innerIgnore(this); //see comments for InnerIgnore class
 
   bool bResult = false;
@@ -401,7 +400,7 @@ bool AddDel::select_item(const Gtk::TreeModel::iterator& iter, guint column, boo
     if(refTreeSelection)
     {
       refTreeSelection->select(iter);
-          
+
       Gtk::TreeModel::Path path = m_refListStore->get_path(iter);
 
       guint view_column_index = 0;
@@ -656,40 +655,46 @@ void AddDel::construct_specified_columns()
       }
 
     ++view_column_index;
-    
+
     } //is visible
 
     ++model_column_index;
   }
-  
+
   //Delete the vector's items:
   for(type_vecModelColumns::iterator iter = vecModelColumns.begin(); iter != vecModelColumns.end(); ++iter)
   {
      Gtk::TreeModelColumnBase* pModelColumn = *iter;
-     switch(m_ColumnTypes[model_column_index].m_style)
-     {
-       //Cast it to the derived type, so we can delete it properly.
-       //This is necessary because TreeModelColumnBase's destructor is not virtual.
-       case(AddDelColumnInfo::STYLE_Boolean):
-       {
-         Gtk::TreeModelColumn<bool>* pModelColumnDerived = static_cast< Gtk::TreeModelColumn<bool>* >(pModelColumn);
-         delete pModelColumnDerived;
-         break;
-       }
-       default:
-       {
-         Gtk::TreeModelColumn<Glib::ustring>* pModelColumnDerived = static_cast< Gtk::TreeModelColumn<Glib::ustring>* >(pModelColumn);
-         delete pModelColumnDerived;
-         break;
-       }
 
-       *iter = 0;
+     if(model_column_index < (int)m_ColumnTypes.size())
+     {
+       AddDelColumnInfo::enumStyles style = m_ColumnTypes[model_column_index].m_style;
+       switch(style)
+       {
+        //Cast it to the derived type, so we can delete it properly.
+        //This is necessary because TreeModelColumnBase's destructor is not virtual.
+        case(AddDelColumnInfo::STYLE_Boolean):
+        {
+          Gtk::TreeModelColumn<bool>* pModelColumnDerived = static_cast< Gtk::TreeModelColumn<bool>* >(pModelColumn);
+          delete pModelColumnDerived;
+          break;
+        }
+        default:
+        {
+          Gtk::TreeModelColumn<Glib::ustring>* pModelColumnDerived = static_cast< Gtk::TreeModelColumn<Glib::ustring>* >(pModelColumn);
+          delete pModelColumnDerived;
+          break;
+        }
+
+        *iter = 0;
+      }
      }
   }
 
   m_TreeView.columns_autosize();
 }
 
+/*
 void AddDel::set_value(const Gtk::TreeModel::iterator& iter, guint col, const Gnome::Gda::Value& value)
 {
   //Different model columns have different types of data:
@@ -708,6 +713,7 @@ void AddDel::set_value(const Gtk::TreeModel::iterator& iter, guint col, const Gn
     }
   }
 }
+*/
 
 void AddDel::set_value(const Gtk::TreeModel::iterator& iter, guint col, const Glib::ustring& strValue)
 {
@@ -792,10 +798,12 @@ void AddDel::set_value(const Gtk::TreeModel::iterator& iter, guint col, bool bVa
   }
 }
 
+/*
 void AddDel::set_value_selected(guint col, const Gnome::Gda::Value& value)
 {
   set_value(get_item_selected(), col, value);
 }
+*/
 
 void AddDel::remove_all_columns()
 {
@@ -819,32 +827,13 @@ guint AddDel::add_column(const AddDelColumnInfo& column_info)
 
   return m_ColumnTypes.size()-1;
 }
-  
+ 
 guint AddDel::add_column(const Glib::ustring& strTitle, AddDelColumnInfo::enumStyles style, bool editable, bool visible)
 {
   //Use the title as the column_id:
   return add_column(strTitle, strTitle, style, editable, visible);
 }
 
-guint AddDel::add_column(const Field& field, AddDelColumnInfo::enumStyles style, bool editable, bool visible)
-{
-  InnerIgnore innerIgnore(this); //Stop on_treeview_columns_changed() from doing anything when it is called just because we add a new column.
-
-  AddDelColumnInfo column_info;
-  column_info.m_name = field.get_title_or_name();
-  column_info.m_style = style; //TODO: //Use field types instead, or make this dependent on them..
-  column_info.m_id = field.get_name();
-  column_info.m_field_type = field.get_glom_type();
-  column_info.m_editable = editable;
-  column_info.m_visible = visible;
-
-  //Make it non-editable if it is auto-generated:
-  if(field.get_field_info().get_auto_increment())
-    column_info.m_editable = false;
-
-  return add_column(column_info);
-}
-  
 guint AddDel::add_column(const Glib::ustring& strTitle, const Glib::ustring& column_id, AddDelColumnInfo::enumStyles style, bool editable, bool visible)
 {
   InnerIgnore innerIgnore(this); //Stop on_treeview_columns_changed() from doing anything when it is called just because we add a new column.
@@ -863,7 +852,7 @@ Glib::ustring AddDel::get_column_field(guint column_index) const
 {
   Glib::ustring result;
   if(column_index < m_ColumnTypes.size())
-    result = m_ColumnTypes[column_index].m_name;
+    result = m_ColumnTypes[column_index].m_id;
 
   return result;
 }
@@ -927,16 +916,6 @@ bool AddDel::get_allow_user_actions() const
   return m_bAllowUserActions;
 }
 
-void AddDel::set_select_text(const Glib::ustring& strVal)
-{
-  m_strSelectText = strVal;
-}
-
-Glib::ustring AddDel::get_select_text() const
-{
-  return m_strSelectText;
-}
-
 void AddDel::set_show_column_titles(bool bVal)
 {
   m_TreeView.set_headers_visible(bVal);
@@ -998,6 +977,12 @@ void AddDel::remove_item(const Gtk::TreeModel::iterator& iter)
     m_refListStore->erase(iter);
 }
 
+void AddDel::remove_item_by_key(const Glib::ustring& strKey)
+{
+  Gtk::TreeModel::iterator iter = get_row(strKey);
+  remove_item(iter);
+}
+
 AddDel::InnerIgnore::InnerIgnore(AddDel* pOuter)
 {
   m_pOuter = pOuter;
@@ -1185,7 +1170,7 @@ void AddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const Gli
           if(!success)
           {
              //Tell the user and offer to revert or try again:
-             bool revert = glom_show_dialog_invalid_date(field_type);
+             bool revert = glom_show_dialog_invalid_data(field_type);
              if(revert)
              {
                //Revert the data:
@@ -1284,27 +1269,30 @@ void AddDel::on_treeview_button_press_event(GdkEventButton* event)
     Gtk::TreeView::Column* pColumn = 0;
     int cell_x = 0;
     int cell_y = 0;  
-    m_TreeView.get_path_at_pos((int)event->x, (int)event->y, path, pColumn, cell_x, cell_y);
-    
-    //Get the row:
-    Gtk::TreeModel::iterator iterRow = m_refListStore->get_iter(path);
-    if(iterRow)
+    bool row_exists = m_TreeView.get_path_at_pos((int)event->x, (int)event->y, path, pColumn, cell_x, cell_y);
+
+    if(row_exists)
     {
-      //Get the column:
-      int tree_col = 0;
-      int col_index = get_count_hidden_system_columns();
-
-      typedef std::vector<Gtk::TreeView::Column*> type_vecTreeViewColumns;
-      type_vecTreeViewColumns vecColumns = m_TreeView.get_columns();
-      for(type_vecTreeViewColumns::const_iterator iter = vecColumns.begin(); iter != vecColumns.end(); iter++)
+      //Get the row:
+      Gtk::TreeModel::iterator iterRow = m_refListStore->get_iter(path);
+      if(iterRow)
       {
-        if(*iter == pColumn)
-          tree_col = col_index; //Found.
-
-        col_index++;
+        //Get the column:
+        int tree_col = 0;
+        int col_index = get_count_hidden_system_columns();
+  
+        typedef std::vector<Gtk::TreeView::Column*> type_vecTreeViewColumns;
+        type_vecTreeViewColumns vecColumns = m_TreeView.get_columns();
+        for(type_vecTreeViewColumns::const_iterator iter = vecColumns.begin(); iter != vecColumns.end(); iter++)
+        {
+          if(*iter == pColumn)
+            tree_col = col_index; //Found.
+  
+          col_index++;
+        }
+      
+        signal_user_activated().emit(iterRow, tree_col);
       }
-    
-      signal_user_activated().emit(iterRow, tree_col);
     }
   }
 
@@ -1356,7 +1344,7 @@ guint AddDel::treeview_append_column(const Glib::ustring& title, Gtk::CellRender
   pViewColumn->set_column_id( (column_id.empty() ? title : column_id) );
 
   //TODO pViewColumn->signal_button_press_event().connect( sigc::mem_fun(*this, &AddDel::on_treeview_columnheader_button_press_event) );
-  
+
   return cols_count;
 }
 
@@ -1506,12 +1494,6 @@ Glib::ustring AddDel::get_value_key(const Gtk::TreeModel::iterator& iter)
   return get_value(iter, m_col_key);
 }
 
-Gnome::Gda::Value AddDel::get_value_key_as_value(const Gtk::TreeModel::iterator& iter)
-{
-  bool parsed = false;
-  return GlomConversions::parse_value(m_key_field.get_glom_type(), get_value_key(iter), parsed);
-}
-
 
 void AddDel::set_value_key(const Gtk::TreeModel::iterator& iter, const Glib::ustring& strValue)
 {
@@ -1526,6 +1508,9 @@ void AddDel::set_value_key(const Gtk::TreeModel::iterator& iter, const Glib::ust
 
 bool AddDel::get_is_placeholder_row(const Gtk::TreeModel::iterator& iter) const
 {
+  if(!iter)
+    return false;
+
   if(!get_is_last_row(iter))
     return false;
 
@@ -1589,10 +1574,6 @@ void AddDel::set_rules_hint(bool val)
   m_TreeView.set_rules_hint(val);
 }
 
-void AddDel::set_key_type(const Field& field)
-{
-  m_key_field = field;
-}
   
       
 
