@@ -27,8 +27,10 @@
 #include "../../dialog_invalid_data.h"
 #include "../../application.h"
 #include "../../utils.h"
+#include "cellrenderer_button.h"
 //#include "../cellrendererlist.h"
 #include <iostream> //For debug output.
+#include <gtk/gtktreeview.h>
 
 DbAddDelColumnInfo::DbAddDelColumnInfo()
 :
@@ -61,7 +63,9 @@ DbAddDel::DbAddDel()
   m_allow_add(true),
   m_allow_delete(true),
   m_columns_ready(false),
-  m_allow_view(true)
+  m_allow_view(true),
+  m_allow_view_details(false),
+  m_treeviewcolumn_button(0)
 {
   set_prevent_user_signals();
   set_ignore_treeview_signals();
@@ -199,7 +203,11 @@ void DbAddDel::setup_menu()
 
   m_refActionGroup->add(Gtk::Action::create("ContextMenu", "Context Menu") );
 
-  m_refContextEdit =  Gtk::Action::create("ContextEdit", Gtk::Stock::EDIT);
+  if(m_open_button_title.empty())
+    m_refContextEdit =  Gtk::Action::create("ContextEdit", Gtk::Stock::EDIT);
+  else
+    m_refContextEdit =  Gtk::Action::create("ContextEdit", m_open_button_title);
+    
   m_refActionGroup->add(m_refContextEdit,
     sigc::mem_fun(*this, &DbAddDel::on_MenuPopup_activate_Edit) );
 
@@ -287,17 +295,22 @@ bool DbAddDel::on_button_press_event_Popup(GdkEventButton *event)
   {
     //Give user choices of actions on this item:
     m_pMenuPopup->popup(event->button, event->time);
+    return true; //handled.
   }
   else
   {
     if(event->type == GDK_2BUTTON_PRESS)
     {
       //Double-click means edit.
-      on_MenuPopup_activate_Edit();
+      //Don't do this usually, because users sometimes double-click by accident when they just want to edit a cell.
+      
+      //TODO: If the cell is not editable, handle the double-click as an edit/selection.
+      //on_MenuPopup_activate_Edit();
+      return false; //Not handled.
     }
   }
 
-  return true;
+  return  false; //Not handled. TODO: Call base class?
 }
 
 Gtk::TreeModel::iterator DbAddDel::get_item_placeholder()
@@ -637,6 +650,19 @@ void DbAddDel::construct_specified_columns()
   //Add new View Colums:
   int model_column_index = 0; //Not including the hidden internal columns.
   int view_column_index = 0;
+  
+  {
+    GlomCellRenderer_Button* pCellButton = Gtk::manage(new GlomCellRenderer_Button());
+    pCellButton->signal_clicked().connect(sigc::mem_fun(*this, &DbAddDel::on_cell_button_clicked));
+    
+    const int viewcols_count = m_TreeView.append_column(Glib::ustring(), *pCellButton);
+    m_treeviewcolumn_button = m_TreeView.get_column(viewcols_count - 1);
+    
+    m_treeviewcolumn_button->property_visible() = get_allow_view_details();
+    
+    ++view_column_index;
+  }
+  
   for(type_vecModelColumns::iterator iter = vecModelColumns.begin(); iter != vecModelColumns.end(); ++iter)
   {
     DbAddDelColumnInfo& column_info = m_ColumnTypes[model_column_index];
@@ -657,10 +683,10 @@ void DbAddDel::construct_specified_columns()
         }
         default:
         {
-          if(column_info.m_field.get_has_choices())
+          if(column_info.m_field.get_formatting_used().get_has_choices())
           {
             CellRendererList* rendererList = Gtk::manage( new CellRendererList() );
-            rendererList->set_restrict_values_to_list(column_info.m_field.get_choices_restricted());
+            rendererList->set_restrict_values_to_list(column_info.m_field.get_formatting_used().get_choices_restricted());
 
             pCellRenderer = rendererList;
           }
@@ -696,25 +722,25 @@ void DbAddDel::construct_specified_columns()
           {
             pCellRendererCombo->remove_all_list_items();
 
-            if(column_info.m_field.get_has_custom_choices())
+            if(column_info.m_field.get_formatting_used().get_has_custom_choices())
             {
               pCellRendererCombo->set_use_second(false); //Custom choices have only one column.
 
               //set_choices() needs this, for the numeric layout:
               //pCellRendererCombo->set_layout_item(get_layout_item()->clone(), table_name); //TODO_Performance: We only need this for the numerical format.
-              const LayoutItem_Field::type_list_values list_values = column_info.m_field.get_choices_custom();
-              for(LayoutItem_Field::type_list_values::const_iterator iter = list_values.begin(); iter != list_values.end(); ++iter)
+              const FieldFormatting::type_list_values list_values = column_info.m_field.get_formatting_used().get_choices_custom();
+              for(FieldFormatting::type_list_values::const_iterator iter = list_values.begin(); iter != list_values.end(); ++iter)
               {
-                pCellRendererCombo->append_list_item( GlomConversions::get_text_for_gda_value(column_info.m_field.m_field.get_glom_type(), *iter, column_info.m_field.m_numeric_format) );
+                pCellRendererCombo->append_list_item( GlomConversions::get_text_for_gda_value(column_info.m_field.m_field.get_glom_type(), *iter, column_info.m_field.get_formatting_used().m_numeric_format) );
               }
             }
-            else if(column_info.m_field.get_has_related_choices())
+            else if(column_info.m_field.get_formatting_used().get_has_related_choices())
             {
               Glib::ustring choice_relationship_name, choice_field, choice_second;
-              column_info.m_field.get_choices(choice_relationship_name, choice_field, choice_second);
+              column_info.m_field.get_formatting_used().get_choices(choice_relationship_name, choice_field, choice_second);
               if(!choice_relationship_name.empty() && !choice_field.empty())
               {
-                const Relationship relationship = column_info.m_field.m_choices_related_relationship;
+                const Relationship relationship = column_info.m_field.get_formatting_used().m_choices_related_relationship;
                 const Glib::ustring to_table = relationship.get_to_table();
 
                 const bool use_second = !choice_second.empty();
@@ -738,11 +764,11 @@ void DbAddDel::construct_specified_columns()
                 GlomUtils::type_list_values_with_second list_values = GlomUtils::get_choice_values(column_info.m_field);
                 for(GlomUtils::type_list_values_with_second::const_iterator iter = list_values.begin(); iter != list_values.end(); ++iter)
                 {
-                  const Glib::ustring first = GlomConversions::get_text_for_gda_value(column_info.m_field.m_field.get_glom_type(), iter->first, column_info.m_field.m_numeric_format);
+                  const Glib::ustring first = GlomConversions::get_text_for_gda_value(column_info.m_field.m_field.get_glom_type(), iter->first, column_info.m_field.get_formatting_used().m_numeric_format);
 
                   Glib::ustring second;
                   if(use_second)
-                    second = GlomConversions::get_text_for_gda_value(layout_field_second.m_field.get_glom_type(), iter->second, layout_field_second.m_numeric_format);
+                    second = GlomConversions::get_text_for_gda_value(layout_field_second.m_field.get_glom_type(), iter->second, layout_field_second.get_formatting_used().m_numeric_format);
 
                   pCellRendererCombo->append_list_item(first, second);
                 }
@@ -791,7 +817,8 @@ bool DbAddDel::refresh_from_database()
   {
     Glib::RefPtr<Gtk::TreeModel> refNull;
     bool result = m_refListStore->refresh_from_database(m_where_clause);
-    m_TreeView.set_model(refNull); //TODO: Find a better way to indicate that the whole model has changed, because this causes a g_waring().
+    m_TreeView.set_model(refNull); //TODO: This causes a g_waring(): gtk_tree_view_unref_tree_helper: assertion `node != NULL' failed
+    //gtk_tree_view_set_model(m_TreeView.gobj(), 0); //This gives the same warning.
     m_TreeView.set_model(m_refListStore);
     return result;
   }
@@ -967,7 +994,7 @@ void DbAddDel::set_allow_add(bool val)
 
 void DbAddDel::set_allow_delete(bool val)
 {
-  m_allow_delete= val;
+  m_allow_delete = val;
 }
   
   
@@ -1211,7 +1238,7 @@ void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const G
     {
       //Make sure that the entered data is suitable for this field type:
       bool success = false;
-      Gnome::Gda::Value value = GlomConversions::parse_value(field_type, new_text, m_ColumnTypes[model_column_index].m_field.m_numeric_format, success);
+      Gnome::Gda::Value value = GlomConversions::parse_value(field_type, new_text, m_ColumnTypes[model_column_index].m_field.get_formatting_used().m_numeric_format, success);
       if(!success)
       {
           //Tell the user and offer to revert or try again:
@@ -1503,7 +1530,6 @@ bool DbAddDel::get_is_last_row(const Gtk::TreeModel::iterator& iter) const
 
 Gtk::TreeModel::iterator DbAddDel::get_next_available_row_with_add_if_necessary()
 {
-g_warning("get_next_available_row_with_add_if_necessary");
   Gtk::TreeModel::iterator result;
 
   if(!m_refListStore)
@@ -1670,7 +1696,7 @@ void DbAddDel::treeviewcolumn_on_cell_data(Gtk::CellRenderer* renderer, const Gt
         //TODO: Maybe we should have custom cellrenderers for time, date, and numbers.
         Gtk::CellRendererText* pDerived = dynamic_cast<Gtk::CellRendererText*>(renderer);
 
-        const Glib::ustring text = GlomConversions::get_text_for_gda_value(column_info.m_field.m_field.get_glom_type(), value, column_info.m_field.m_numeric_format);
+        const Glib::ustring text = GlomConversions::get_text_for_gda_value(column_info.m_field.m_field.get_glom_type(), value, column_info.m_field.get_formatting_used().m_numeric_format);
         //g_assert(text != "NULL");
         pDerived->property_text() = text;
 
@@ -1693,5 +1719,37 @@ void DbAddDel::set_allow_view(bool val)
   m_allow_view = val;
 }
 
+void DbAddDel::set_allow_view_details(bool val)
+{
+  m_allow_view_details = val;
+  
+  //Hide it if it was visible:
+  if(m_treeviewcolumn_button)
+    m_treeviewcolumn_button->property_visible() = get_allow_view_details();
+    
+}
+
+bool DbAddDel::get_allow_view_details() const
+{
+  return m_allow_view_details;
+}
+
+void DbAddDel::on_cell_button_clicked(const Gtk::TreeModel::Path& path)
+{
+  Gtk::TreeModel::iterator iter = m_refListStore->get_iter(path);
+  if(iter)
+  {
+    const guint first_visible = get_count_hidden_system_columns();
+    select_item(iter, first_visible, false /* start_editing */);
+  }
+  
+  on_MenuPopup_activate_Edit();
+}
+
+void DbAddDel::set_open_button_title(const Glib::ustring& title)
+{
+  m_open_button_title = title;
+}
+
 
 
