@@ -135,6 +135,11 @@ public:
 
   ///For instance, if the user can not view the table then don't try to get the records.
   virtual void set_allow_view(bool val = true);
+  
+  ///Whether each row should have a button, to request edit.
+  virtual void set_allow_view_details(bool val = true);
+  bool get_allow_view_details() const;
+
 
   /** @result The index of the new column.
    */
@@ -223,6 +228,8 @@ public:
 
   virtual Gtk::TreeModel::iterator get_last_row();
   virtual Gtk::TreeModel::iterator get_last_row() const;
+  
+  virtual void set_open_button_title(const Glib::ustring& title);
 
 protected:
 
@@ -262,7 +269,7 @@ protected:
   virtual void on_treeview_button_press_event(GdkEventButton* event);
 
   virtual bool on_treeview_columnheader_button_press_event(GdkEventButton* event);
-
+  virtual void on_cell_button_clicked(const Gtk::TreeModel::Path& path);  
 
   bool get_prevent_user_signals() const;
 
@@ -300,6 +307,7 @@ protected:
   typedef std::vector<DbAddDelColumnInfo> type_ColumnTypes;
   type_ColumnTypes m_ColumnTypes;
   Glib::ustring m_table_name, m_where_clause;
+  Glib::ustring m_open_button_title; //Allow us to change "Open" to "Select".
 
   Gtk::Menu* m_pMenuPopup;
   Glib::RefPtr<Gtk::ActionGroup> m_refActionGroup;
@@ -321,6 +329,8 @@ protected:
 
   bool m_columns_ready;
   bool m_allow_view;
+  bool m_allow_view_details;
+  Gtk::TreeViewColumn* m_treeviewcolumn_button;
 
   //signals:
   type_signal_user_added m_signal_user_added;
